@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
-    2012, 2013 Antonio Diaz Diaz.
+    Copyright (C) 2003-2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -41,7 +40,7 @@ Bitmap::Bitmap( const Bitmap & source, const Rectangle & re )
   : Rectangle( re ), data( re.height() )
   {
   if( !source.includes( re ) )
-    Ocrad::internal_error( "bad parameter building a Bitmap from part of another one" );
+    Ocrad::internal_error( "bad parameter building a Bitmap from part of another one." );
 
   const int ldiff = left()-source.left();
   const int tdiff = top()-source.top();
