@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
-    Antonio Diaz Diaz.
+    Copyright (C) 2003-2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -18,7 +17,7 @@
 
 class Bitmap : public Rectangle
   {
-  std::vector< std::vector< unsigned char > > data;	// faster than bool
+  std::vector< std::vector< uint8_t > > data;		// faster than bool
 
 public:
       // Creates a blank Bitmap
@@ -46,22 +45,22 @@ public:
   bool adjust_height();
   bool adjust_width();
 
-  bool get_bit( const int row, const int col ) const throw()
+  bool get_bit( const int row, const int col ) const
     { return data[row-top()][col-left()]; }
-  void set_bit( const int row, const int col, const bool bit ) throw()
+  void set_bit( const int row, const int col, const bool bit )
     { data[row-top()][col-left()] = bit; }
 
-  int area() const throw();			// 'area' means filled area
-  int area_octagon() const throw();
-  int size_octagon() const throw();
-  int seek_left  ( const int row, const int col, const bool black = true ) const throw();
-  int seek_top   ( const int row, const int col, const bool black = true ) const throw();
-  int seek_right ( const int row, const int col, const bool black = true ) const throw();
-  int seek_bottom( const int row, const int col, const bool black = true ) const throw();
-  bool escape_left  ( int row, int col ) const throw();
-  bool escape_top   ( int row, int col ) const throw();
-  bool escape_right ( int row, int col ) const throw();
-  bool escape_bottom( int row, int col ) const throw();
+  int area() const;			// 'area' means filled area
+  int area_octagon() const;
+  int size_octagon() const;
+  int seek_left  ( const int row, const int col, const bool black = true ) const;
+  int seek_top   ( const int row, const int col, const bool black = true ) const;
+  int seek_right ( const int row, const int col, const bool black = true ) const;
+  int seek_bottom( const int row, const int col, const bool black = true ) const;
+  bool escape_left  ( int row, int col ) const;
+  bool escape_top   ( int row, int col ) const;
+  bool escape_right ( int row, int col ) const;
+  bool escape_bottom( int row, int col ) const;
   int  follow_top   ( int row, int col ) const;
   int  follow_bottom( int row, int col ) const;
   bool top_hook   ( int *hdiff ) const;
