@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
-    Antonio Diaz Diaz.
+    Copyright (C) 2003-2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -19,31 +18,35 @@
 #include <algorithm>
 #include <cstdio>
 #include <vector>
+#include <stdint.h>
 
 #include "common.h"
 #include "rectangle.h"
+#include "segment.h"
 #include "ucs.h"
 #include "bitmap.h"
 #include "blob.h"
 #include "character.h"
+#include "profile.h"
+#include "feats.h"
 
 
-Character::Character( const Character & c ) throw()
+Character::Character( const Character & c )
   : Rectangle( c ), blobpv( c.blobpv ), gv( c.gv )
   {
-  for( unsigned int i = 0; i < blobpv.size(); ++i )
+  for( unsigned i = 0; i < blobpv.size(); ++i )
     blobpv[i] = new Blob( *c.blobpv[i] );
   }
 
 
-Character & Character::operator=( const Character & c ) throw()
+Character & Character::operator=( const Character & c )
   {
   if( this != &c )
     {
     Rectangle::operator=( c );
-    for( unsigned int i = 0; i < blobpv.size(); ++i ) delete blobpv[i];
+    for( unsigned i = 0; i < blobpv.size(); ++i ) delete blobpv[i];
     blobpv = c.blobpv;
-    for( unsigned int i = 0; i < blobpv.size(); ++i )
+    for( unsigned i = 0; i < blobpv.size(); ++i )
       blobpv[i] = new Blob( *c.blobpv[i] );
     gv = c.gv;
     }
@@ -51,15 +54,15 @@ Character & Character::operator=( const Character & c ) throw()
   }
 
 
-Character::~Character() throw()
+Character::~Character()
   {
-  for( unsigned int i = 0; i < blobpv.size(); ++i ) delete blobpv[i];
+  for( unsigned i = 0; i < blobpv.size(); ++i ) delete blobpv[i];
   }
 
 
 // Return the filled area of the main blobs only (no recursive)
 //
-int Character::area() const throw()
+int Character::area() const
   {
   int a = 0;
   for( int i = 0; i < blobs(); ++i ) a += blobpv[i]->area();
@@ -67,7 +70,15 @@ int Character::area() const throw()
   }
 
 
-Blob & Character::blob( int i ) throw()
+const Blob & Character::blob( const int i ) const
+  {
+  if( i < 0 || i >= blobs() )
+    Ocrad::internal_error( "const blob, index out of bounds" );
+  return *blobpv[i];
+  }
+
+
+Blob & Character::blob( const int i )
   {
   if( i < 0 || i >= blobs() )
     Ocrad::internal_error( "blob, index out of bounds" );
@@ -75,7 +86,7 @@ Blob & Character::blob( int i ) throw()
   }
 
 
-Blob & Character::main_blob() throw()
+Blob & Character::main_blob()
   {
   int imax = 0;
   for( int i = 1; i < blobs(); ++i )
@@ -85,7 +96,7 @@ Blob & Character::main_blob() throw()
   }
 
 
-void Character::shift_blobp( Blob * p ) throw()
+void Character::shift_blobp( Blob * const p )
   {
   add_rectangle( *p );
   int i = blobs() - 1;
@@ -95,17 +106,11 @@ void Character::shift_blobp( Blob * p ) throw()
     if( p->vcenter() > bi.vcenter() ) break;
     if( p->vcenter() == bi.vcenter() && p->hcenter() >= bi.hcenter() ) break;
     }
-  blobpv.insert( blobpv.begin() + i + 1, p );
+  blobpv.insert( blobpv.begin() + ( i + 1 ), p );
   }
 
 
-void Character::add_guess( int code, int value ) throw()
-  {
-  gv.push_back( Guess( code, value ) );
-  }
-
-
-void Character::insert_guess( int i, int code, int value ) throw()
+void Character::insert_guess( const int i, const int code, const int value )
   {
   if( i < 0 || i > guesses() )
     Ocrad::internal_error( "insert_guess, index out of bounds" );
@@ -113,7 +118,7 @@ void Character::insert_guess( int i, int code, int value ) throw()
   }
 
 
-void Character::delete_guess( int i ) throw()
+void Character::delete_guess( const int i )
   {
   if( i < 0 || i >= guesses() )
     Ocrad::internal_error( "delete_guess, index out of bounds" );
@@ -121,20 +126,32 @@ void Character::delete_guess( int i ) throw()
   }
 
 
-void Character::only_guess( int code, int value ) throw()
-  { gv.clear(); add_guess( code, value ); }
+bool Character::set_merged_guess( const int code1, const int right1,
+                                  const int code2, const int blob_index )
+  {
+  if( blob_index < 0 || blob_index >= blobs() ) return false;
+  const Blob & b = *blobpv[blob_index];
+  if( b.left() <= right1 && right1 < b.right() )
+    {
+    only_guess( -(blob_index + 1), left() );
+    add_guess( code1, right1 );
+    add_guess( code2, right() );
+    return true;
+    }
+  return false;
+  }
 
 
-void Character::swap_guesses( int i, int j ) throw()
+void Character::swap_guesses( const int i, const int j )
   {
   if( i < 0 || i >= guesses() || j < 0 || j >= guesses() )
     Ocrad::internal_error( "swap_guesses, index out of bounds" );
-  int code = gv[i].code;
+  const int code = gv[i].code;
   gv[i].code = gv[j].code; gv[j].code = code;
   }
 
 
-const Character::Guess & Character::guess( int i ) const throw()
+const Character::Guess & Character::guess( const int i ) const
   {
   if( i < 0 || i >= guesses() )
     Ocrad::internal_error( "guess, index out of bounds" );
@@ -142,7 +159,7 @@ const Character::Guess & Character::guess( int i ) const throw()
   }
 
 
-bool Character::maybe( int code ) const throw()
+bool Character::maybe( const int code ) const
   {
   for( int i = 0; i < guesses(); ++i )
     if( code == gv[i].code ) return true;
@@ -150,7 +167,7 @@ bool Character::maybe( int code ) const throw()
   }
 
 /*
-bool Character::maybe_digit() const throw()
+bool Character::maybe_digit() const
   {
   for( int i = 0; i < guesses(); ++i )
     if( UCS::isdigit( gv[i].code ) ) return true;
@@ -158,7 +175,7 @@ bool Character::maybe_digit() const throw()
   }
 
 
-bool Character::maybe_letter() const throw()
+bool Character::maybe_letter() const
   {
   for( int i = 0; i < guesses(); ++i )
     if( UCS::isalpha( gv[i].code ) ) return true;
@@ -166,14 +183,14 @@ bool Character::maybe_letter() const throw()
   }
 */
 
-void Character::join( Character & c ) throw()
+void Character::join( Character & c )
   {
   for( int i = 0; i < c.blobs(); ++i ) shift_blobp( c.blobpv[i] );
   c.blobpv.clear();
   }
 
 
-unsigned char Character::byte_result() const throw()
+unsigned char Character::byte_result() const
   {
   if( guesses() )
     {
@@ -184,44 +201,52 @@ unsigned char Character::byte_result() const throw()
   }
 
 
-void Character::print( const Control & control ) const throw()
+const char * Character::utf8_result() const
   {
   if( guesses() )
     {
-//    if( maybe( '\t' ) ) std::putc( '\t', control.outfile ); else
-    switch( control.format )
+    const char * s = UCS::ucs_to_utf8( gv[0].code );
+    if( *s ) return s;
+    }
+  return "_";
+  }
+
+
+void Character::print( const Control & control ) const
+  {
+  if( guesses() )
+    {
+    if( !control.utf8 )
       {
-      case Control::byte:
-        { unsigned char ch = UCS::map_to_byte( gv[0].code );
-        if( ch ) std::putc( ch, control.outfile ); }
-        break;
-      case Control::utf8:
-        if( gv[0].code )
-          std::fputs( UCS::ucs_to_utf8( gv[0].code ), control.outfile );
+      unsigned char ch = UCS::map_to_byte( gv[0].code );
+      if( ch ) std::putc( ch, control.outfile );
       }
+    else if( gv[0].code )
+      std::fputs( UCS::ucs_to_utf8( gv[0].code ), control.outfile );
     }
   else std::putc( '_', control.outfile );
   }
 
 
 void Character::dprint( const Control & control, const Rectangle & charbox,
-                        bool graph, bool recursive ) const throw()
+                        const bool graph, const bool recursive ) const
   {
   if( graph || recursive )
     std::fprintf( control.outfile, "%d guesses    ", guesses() );
   for( int i = 0; i < guesses(); ++i )
     {
-    switch( control.format )
+    if( gv[i].code == '\t' )
+      std::fprintf( control.outfile, "guess '\\t', confidence %d    ",
+                             gv[i].value );
+    else if( !control.utf8 )
       {
-      case Control::byte:
-        { unsigned char ch = UCS::map_to_byte( gv[i].code );
-        if( ch ) std::fprintf( control.outfile, "guess '%c', confidence %d    ",
-                               ch, gv[i].value ); }
-        break;
-      case Control::utf8:
-        std::fprintf( control.outfile, "guess '%s', confidence %d    ",
-                      UCS::ucs_to_utf8( gv[i].code ), gv[i].value );
+      unsigned char ch = UCS::map_to_byte( gv[i].code );
+      if( ch ) std::fprintf( control.outfile, "guess '%c', confidence %d    ",
+                             ch, gv[i].value );
       }
+    else
+      std::fprintf( control.outfile, "guess '%s', confidence %d    ",
+                    UCS::ucs_to_utf8( gv[i].code ), gv[i].value );
     if( !graph && !recursive ) break;
     }
   std::fputs( "\n", control.outfile );
@@ -231,8 +256,16 @@ void Character::dprint( const Control & control, const Rectangle & charbox,
                   "left = %d, top = %d, right = %d, bottom = %d\n",
                   left(), top(), right(), bottom() );
     std::fprintf( control.outfile,
-                  "width = %d, height = %d, hcenter = %d, vcenter = %d, black area = %d%%\n\n",
+                  "width = %d, height = %d, hcenter = %d, vcenter = %d, black area = %d%%\n",
                   width(), height(), hcenter(), vcenter(), ( area() * 100 ) / size() );
+    if( blobs() >= 1 && blobs() <= 3 )
+      {
+      const Blob & b = blob( blobs() - 1 );
+      Features f( b );
+      std::fprintf( control.outfile,
+                    "hbars = %d, vbars = %d\n", f.hbars(), f.vbars() );
+      }
+    std::fputs( "\n", control.outfile );
 
     const int minrow = std::min( top(), charbox.top() );
     const int maxrow = std::max( bottom(), charbox.bottom() );
@@ -260,7 +293,7 @@ void Character::dprint( const Control & control, const Rectangle & charbox,
 
       for( int col = left(); col <= right(); ++col )
         {
-        char ch = ( isvc && col == hcenter() ) ? '+' : '';
+        char ch = ( isvc && col == hcenter() ) ? '+' : '.';
         for( int i = 0; i < blobs(); ++i )
           {
           int id = blobpv[i]->id( row, col );
@@ -293,61 +326,73 @@ void Character::dprint( const Control & control, const Rectangle & charbox,
   }
 
 
-void Character::xprint( const Control & control ) const throw()
+void Character::xprint( const Control & control ) const
   {
   std::fprintf( control.exportfile, "%3d %3d %2d %2d; %d",
                 left(), top(), width(), height(), guesses() );
 
   for( int i = 0; i < guesses(); ++i )
-    switch( control.format )
+    if( !control.utf8 )
       {
-      case Control::byte:
-        { unsigned char ch = UCS::map_to_byte( gv[i].code );
-        if( !ch ) ch = '_';
-        std::fprintf( control.exportfile, ", '%c'%d", ch, gv[i].value ); }
-        break;
-      case Control::utf8:
-        std::fprintf( control.exportfile, ", '%s'%d",
-                      UCS::ucs_to_utf8( gv[i].code ), gv[i].value );
+      unsigned char ch = UCS::map_to_byte( gv[i].code );
+      if( !ch ) ch = '_';
+      std::fprintf( control.exportfile, ", '%c'%d", ch, gv[i].value );
       }
+    else
+      std::fprintf( control.exportfile, ", '%s'%d",
+                    UCS::ucs_to_utf8( gv[i].code ), gv[i].value );
   std::fputs( "\n", control.exportfile );
   }
 
 
-void Character::apply_filter( const Filter & filter ) throw()
+void Character::apply_filter( const Filter::Type filter )
   {
-  if( filter.type() == Filter::none ) return;
-  const int code = guesses() ? gv[0].code : 0;
+  if( !guesses() ) return;
+  const int code = gv[0].code;
   bool remove = false;
 
-  switch( filter.type() )
+  switch( filter )
     {
-    case Filter::none:			// only for completeness
-      break;
     case Filter::letters_only:
-      remove = true;
+      remove = true;				// fall through
     case Filter::letters:
       if( !UCS::isalpha( code ) && !UCS::isspace( code ) )
         {
         for( int i = 1; i < guesses(); ++i )
           if( UCS::isalpha( gv[i].code ) ) { swap_guesses( 0, i ); break; }
-        if( guesses() && !UCS::isalpha( gv[0].code ) )
+        if( gv[0].code == '+' && 2 * height() > 3 * width() )
+          { gv[0].code = 't'; break; }
+        if( !UCS::isalpha( gv[0].code ) )
           gv[0].code = UCS::to_nearest_letter( gv[0].code );
-        if( remove && ( !guesses() || !UCS::isalpha( gv[0].code ) ) )
-          only_guess( 0, 0 );
+        if( remove && !UCS::isalpha( gv[0].code ) ) clear_guesses();
         }
       break;
     case Filter::numbers_only:
-      remove = true;
+      remove = true;				// fall through
     case Filter::numbers:
       if( !UCS::isdigit( code ) && !UCS::isspace( code ) )
         {
         for( int i = 1; i < guesses(); ++i )
           if( UCS::isdigit( gv[i].code ) ) { swap_guesses( 0, i ); break; }
-        if( guesses() && !UCS::isdigit( gv[0].code ) )
+        if( !UCS::isdigit( gv[0].code ) )
           gv[0].code = UCS::to_nearest_digit( gv[0].code );
-        if( remove && ( !guesses() || !UCS::isdigit( gv[0].code ) ) )
-          only_guess( 0, 0 );
+        if( remove && !UCS::isdigit( gv[0].code ) ) clear_guesses();
+        }
+      break;
+    case Filter::same_height: break;		// handled at line level
+    case Filter::upper_num_only:
+      remove = true;				// fall through
+    case Filter::upper_num:
+      if( !UCS::isupper( code ) && !UCS::isdigit( code ) &&
+          !UCS::isspace( code ) )
+        {
+        for( int i = 1; i < guesses(); ++i )
+          if( UCS::isupper( gv[i].code ) || UCS::isdigit( gv[i].code ) )
+            { swap_guesses( 0, i ); break; }
+        if( !UCS::isupper( gv[0].code ) && !UCS::isdigit( gv[0].code ) )
+          gv[0].code = UCS::to_nearest_upper_num( gv[0].code );
+        if( remove && !UCS::isupper( gv[0].code ) && !UCS::isdigit( gv[0].code ) )
+          clear_guesses();
         }
       break;
     }
