@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
-    Antonio Diaz Diaz.
+    Copyright (C) 2003-2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -18,10 +17,13 @@
 
 #include <algorithm>
 #include <cstdio>
+#include <cstdlib>
 #include <vector>
+#include <stdint.h>
 
 #include "common.h"
 #include "rectangle.h"
+#include "segment.h"
 #include "ucs.h"
 #include "bitmap.h"
 #include "blob.h"
@@ -34,25 +36,18 @@
 //
 void Character::recognize1( const Charset & charset, const Rectangle & charbox )
   {
-  if( blobs() == 1 ) recognize11( charset, charbox );
+  if( blobs() == 1 )
+    {
+    const Blob & b = blob( 0 );
+    if( b.holes() == 0 ) recognize110( charset, charbox );
+    else if( b.holes() == 1 ) recognize111( charset, charbox );
+    else if( b.holes() == 2 ) recognize112( charbox );
+    }
   else if( blobs() == 2 ) recognize12( charset, charbox );
   else if( blobs() == 3 ) recognize13( charset, charbox );
   }
 
 
-// Recognizes 1 blob characters.
-// 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghklmnopqrstuvwxyz
-// #$&'()*+,-./<>@[\]^_`{|}~
-//
-void Character::recognize11( const Charset & charset, const Rectangle & charbox )
-  {
-  const Blob & b = blob( 0 );
-  if( b.holes() == 0 ) recognize110( charset, charbox );
-  else if( b.holes() == 1 ) recognize111( charset, charbox );
-  else if( b.holes() == 2 ) recognize112( charbox );
-  }
-
-
 // Recognizes 1 blob characters without holes.
 // 12357CEFGHIJKLMNSTUVWXYZcfhklmnrstuvwxyz
 // '()*+,-./<>@[\]^_`{|}~
@@ -71,13 +66,14 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
   if( b.height() < 5 || ( b.height() < 8 && b.width() < 6 ) ||
       b.height() > 10 * b.width() || 5 * b.height() < b.width() ) return;
 
-  code = f.test_CEFIJLlT( charset ); if( code ) { add_guess( code, 0 ); return; }
-  code = f.test_frst( charbox );     if( code ) { add_guess( code, 0 ); return; }
-  code = f.test_G();                 if( code ) { add_guess( code, 0 ); return; }
-  code = f.test_c();                 if( code ) { add_guess( code, 0 ); return; }
+  code = f.test_EFIJLlT( charset, charbox );
+  if( code ) { add_guess( code, 0 ); return; }
+  code = f.test_frst( charbox );   if( code ) { add_guess( code, 0 ); return; }
+  code = f.test_G();               if( code ) { add_guess( code, 0 ); return; }
+  code = f.test_c();               if( code ) { add_guess( code, 0 ); return; }
   if( charset.enabled( Charset::iso_8859_9 ) )
-    { code = f.test_s_cedilla();     if( code ) { add_guess( code, 0 ); return; } }
-  code = f.test_235Esz( charset );   if( code ) { add_guess( code, 0 ); return; }
+    { code = f.test_s_cedilla();   if( code ) { add_guess( code, 0 ); return; } }
+  code = f.test_235Esz( charset ); if( code ) { add_guess( code, 0 ); return; }
   code = f.test_HKMNUuvwYy( charbox );
   if( code == 'u' && f.lp.istpit() )	// Looks for merged 'tr'
     {
@@ -106,7 +102,7 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
         ( b.height() >= 16 && f.bp.minima( b.height() / 8 ) == 2 ) ) )
     {
     code = f.test_hknwx( charbox );
-    if( code == 'n' )	// Looks for '"' or merged 'rr' or 'TT' or 'rt' or 'fl'
+    if( code == 'n' )	// Looks for '"' or merged 'rt' or 'fl'
       {
       if( b.bottom() <= charbox.vcenter() ) { add_guess( '"', 0 ); return; }
       if( b.width() > b.height() && 10 * f.lp[f.lp.pos(10)] < b.width() &&
@@ -114,13 +110,7 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
         {
         const int rgap = f.rp[f.rp.pos(50)];
         if( 10 * rgap > b.width() && !b.escape_top( b.vcenter(), b.right() ) )
-          {
-          const int lgap = f.lp[f.lp.pos(50)];
-          if( 10 * lgap > b.width() && !b.escape_top( b.vcenter(), b.left() ) &&
-              Ocrad::similar( lgap, rgap, 10 ) && b.top() < charbox.top() )
-            { if( set_merged_guess( 'T', b.hcenter(), 'T', 0 ) ) return; }
-          else if( set_merged_guess( 'r', b.hcenter(), 'r', 0 ) ) return;
-          }
+          return;	// leave 'rr', 'TT', 'rz', 'FT' etc, for next pass
         }
       if( 2 * f.lp[f.lp.pos(10)] > b.width() && !f.rp.increasing( f.rp.pos( 75 ) ) )
         {
@@ -206,9 +196,12 @@ void Character::recognize110( const Charset & charset, const Rectangle & charbox
 
   if( f.tp.minima( b.height() / 4 ) == 3 )
     {
+    if( f.segments_in_row( b.vcenter() ) == 2 &&
+        f.segments_in_row( b.vpos( 80 ) ) == 3 ) return;	// merged 'IX'
     int hdiff;
     if( !b.bottom_hook( &hdiff ) &&
-        ( f.vscan()[b.vcenter() - b.top()] < 4 || !b.escape_top( b.vcenter(), b.hcenter() ) ) )
+        ( f.segments_in_row( b.vcenter() ) < 4 ||
+          !b.escape_top( b.vcenter(), b.hcenter() ) ) )
       add_guess( 'w', 0 );
     return;
     }
@@ -325,8 +318,9 @@ void Character::recognize112( const Rectangle & charbox )
       if( b.bottom_hook( &hdiff ) && hdiff > b.height() / 2 )
         if( b.top_hook( &hdiff ) && hdiff > b.height() / 2 )
           { add_guess( 's', 0 ); return; }
-    
-      if( lp.isflats() && !lp.ispit() ) { add_guess( 'B', 0 ); return; }
+
+      if( lp.isflats() && ( lp.istip() || ( lp.isflat() && b.test_BD() ) ) )
+        { add_guess( 'B', 0 ); return; }
 
       int col1 = h1.seek_left( h1.bottom(), h1.right() + 1 ) - 1;
       int col2 = h2.seek_right( h2.top(), h2.left() - 1 ) + 1;
