@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
-    Antonio Diaz Diaz.
+    Copyright (C) 2003-2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -18,10 +17,13 @@
 
 #include <algorithm>
 #include <cstdio>
+#include <cstdlib>
 #include <vector>
+#include <stdint.h>
 
 #include "common.h"
 #include "rectangle.h"
+#include "segment.h"
 #include "ucs.h"
 #include "bitmap.h"
 #include "blob.h"
@@ -32,12 +34,12 @@
 // Looks for three black sections in column hcenter()  n, then tests if
 // upper and lower gaps are open to the right or to the left
 //
-int Features::test_235Esz( const Charset & charset ) const throw()
+int Features::test_235Esz( const Charset & charset ) const
   {
   const int csize = 3;
   const int ucoff[csize] = { 0, -1, +1 };
   const int lcoff[3*csize] = { 0, -1, +1, -1, 0, +1, +1, 0, -1 };
-  const Blob & b = *blob_;
+
   if( b.width() < 9 || b.height() > 3 * b.width() ||
       bp.minima( b.height() / 2 ) > 1 ) return 0;
 
@@ -134,7 +136,8 @@ int Features::test_235Esz( const Charset & charset ) const throw()
               else return UCS::SCCEDI; }
           return 'F';
           }
-        else if( lrow1 < urow2 && lrow2 < urow3 ) return 'E';
+        else if( lrow1 < urow2 && urow2 + 2 < lrow2 && lrow2 < urow3 &&
+                 urow2 <= b.vcenter() && lrow2 >= b.vcenter() ) return 'E';
         }
       }
     else if( b.escape_left( urow2, ucol2 ) )
@@ -142,7 +145,8 @@ int Features::test_235Esz( const Charset & charset ) const throw()
       if( !tbopen && ( 2 * lp[lp.pos(50)] ) + 2 >= b.width() &&
           ( tp.isconvex() || ( tp.ispit() && !bp.ispit() ) ) )
         return '2';
-      if( b.height() <= 2 * wp.max() && bp[bp.pos(75)] <= b.height() / 10 )
+      if( 2 * b.height() <= 5 * wp.max() && bp[bp.pos(75)] <= b.height() / 10 &&
+          Ocrad::similar( wp.max( 0, wp.pos(30) ), wp.max( wp.pos(70) ), 20 ) )
         return 'z';
       }
     }
@@ -150,27 +154,28 @@ int Features::test_235Esz( const Charset & charset ) const throw()
   }
 
 
-int Features::test_CEFIJLlT( const Charset & charset ) const throw()
+int Features::test_EFIJLlT( const Charset & charset, const Rectangle & charbox ) const
   {
-  const Blob & b = *blob_;
-
   if( tp.minima( b.height() / 4 ) != 1 || bp.minima( b.height() / 4 ) != 1 )
     return 0;
 
   const int noise = ( std::min( b.height(), b.width() ) / 30 ) + 1;
   {
-  int col;
-  if( 2 * ( lp[lp.pos(50)] + noise ) >= b.width() ) col = b.hpos( 25 );
-  else col = b.hpos( 75 );
+  const bool maybe_j = ( 2 * ( lp[lp.pos(50)] + noise ) >= b.width() );
+  const int col = b.hpos( maybe_j ? 25 : 75 );
   int row = b.seek_top( b.vcenter(), col );
   if( row <= b.top() || ( row < b.vpos( 25 ) && b.escape_top( row, col ) ) )
     {
     int hdiff;
     if( b.bottom_hook( &hdiff ) )
       {
-      if( hdiff > b.height() / 2 && rp.increasing( rp.pos( 80 ) ) ) return 'J';
-      if( -hdiff > b.height() / 2 )
-        { if( 5 * lp[lp.pos(80)] < 2 * b.width() ) return 'L'; else return 'v'; }
+      if( maybe_j && hdiff > b.height() / 2 &&
+          rp.increasing( rp.pos( 80 ), 1 ) && !rp.decreasing() ) return 'J';
+      if( !maybe_j && -hdiff > b.height() / 2 )
+        {
+        if( 5 * lp[lp.pos(80)] >= 2 * b.width() ) return 'v';	// broken 'v'
+        if( col > b.hcenter() ) return 'L';
+        }
       }
     }
   }
@@ -178,16 +183,18 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
   const int vnoise = ( b.height() / 30 ) + 1;
   const int topmax = b.top() + vnoise;
   const int botmin = b.bottom() - vnoise;
-  if( vbars() == 1 && vbar(0).width() >= 2 && 2 * vbar(0).width() < b.width() )
+  if( vbars() == 1 && vbar(0).width() >= 2 && 2 * vbar(0).width() <= b.width() )
     {
     if( std::abs( vbar(0).hcenter() - b.hcenter() ) <= noise &&
         std::abs( (vbar(0).left() - b.left()) - (b.right() - vbar(0).right()) ) <= 2 * noise )
       {
       if( hbars() == 1 && 4 * hbar(0).height() <= b.height() )
         {
-        if( hbar(0).top() <= topmax || hbar(0).bottom() < b.vpos( 15 ) )
+        if( ( hbar(0).top() <= topmax || hbar(0).bottom() < b.vpos( 15 ) ) &&
+            hbar(0).width() >= wp[wp.pos(75)] + wp[wp.pos(80)] )
           return 'T';
-        if( std::abs( hbar(0).vcenter() - b.vcenter() ) <= 1 &&
+        if( std::abs( hbar(0).vcenter() - b.vcenter() ) <= vnoise &&
+            hbar(0).width() >= b.width() &&
             Ocrad::similar( b.height(), b.width(), 50 ) ) return '+';
         }
       if( hbars() == 2 &&
@@ -209,7 +216,7 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
           vbar(0).h_overlaps( hbar(1) ) )
         {
         if( hbars() == 3 &&
-            Ocrad::similar( hbar(0).width(), hbar(2).width(), 10 ) &&
+            Ocrad::similar( hbar(0).width(), hbar(2).width(), 10, 2 ) &&
             10 * hbar(2).width() >= 9 * hbar(1).width() &&
             hbar(0).left() <= hbar(1).left() + 1 )
           return 'E';
@@ -238,14 +245,20 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
         }
       }
 
-    if( vbar(0).left() > b.hcenter() &&
-        hbars() == 1 && hbar(0).top() <= topmax &&
-        hbar(0).width() + 1 >= b.width() )
+    if( vbar(0).left() > b.hcenter() && hbars() == 1 )
       {
-      if( charset.enabled( Charset::iso_8859_15 ) ||
-          charset.enabled( Charset::iso_8859_9 ) )
-        if( b.width() > b.height() ) return UCS::NOT;
-      return 0;
+      if( vbar(0).right() >= b.hpos( 90 ) && hbar(0).bottom() >= botmin &&
+          Ocrad::similar( hbar(0).width(), b.width(), 10 ) &&
+          b.bottom() > charbox.vpos( 90 ) &&
+          b.escape_top( b.vcenter(), b.hpos( 25 ) ) )
+        { if( b.height() > b.width() ) return 'J'; else return 0; }
+      if( hbar(0).top() <= topmax && hbar(0).width() + 1 >= b.width() )
+        {
+        if( charset.enabled( Charset::iso_8859_15 ) ||
+            charset.enabled( Charset::iso_8859_9 ) )
+          if( b.width() > b.height() ) return UCS::NOT;
+        return 0;
+        }
       }
     }
 
@@ -284,22 +297,6 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
               if( b.seek_bottom( b.vpos( 75 ), i ) < b.bottom() &&
                   bp[i-b.left()] <= noise ) return 'l';
       }
-    if( vbar(0).left() <= b.left() + 1 && b.height() > 2 * b.width() &&
-        rp.istip() )
-      {
-      if( 2 * rp[rp.pos(50)] > b.width() )
-        {
-        int row = b.seek_top( b.vcenter(), b.hcenter() );
-        int col = b.seek_right( row, b.hcenter() );
-        if( col < b.right() )
-          {
-          row = b.seek_bottom( b.vcenter(), b.hcenter() );
-          col = b.seek_right( row, b.hcenter() );
-          if( col < b.right() ) return 'C';
-          }
-        }
-      return '[';
-      }
     if( vbar(0).right() >= b.right() - 1 )
       {
       if( lp.istip() && b.height() > 2 * b.width() )
@@ -315,7 +312,8 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
           if( i > lp.pos( 10 ) && i < lp.pos( 40 ) ) return '1'; }
       }
     }
-  if( hbars() == 1 && std::abs( hbar(0).vcenter() - b.vcenter() ) <= 1 &&
+  if( hbars() == 1 && hbar(0).width() >= b.width() &&
+      std::abs( hbar(0).vcenter() - b.vcenter() ) <= vnoise &&
       Ocrad::similar( b.height(), b.width(), 50 ) &&
       tp.isupit() && bp.isupit() )
     return '+';
@@ -323,22 +321,18 @@ int Features::test_CEFIJLlT( const Charset & charset ) const throw()
   }
 
 
-int Features::test_c() const throw()
+int Features::test_c() const
   {
-  const Blob & b = *blob_;
-
   if( lp.isconvex() || lp.ispit() )
     {
     int urow = b.seek_top( b.vcenter(), b.hcenter() );
     int lrow = b.seek_bottom( b.vcenter(), b.hcenter() );
 
-    if( b.height() > 2 * b.width() &&
-        ( 3 * wp.max() <= 2 * b.width() ||
-        ( 2 * rp[urow-b.top()] >= b.width() && 2 * rp[lrow-b.top()] >= b.width() ) ) )
+    if( b.height() > 2 * b.width() && 3 * wp.max() <= 2 * b.width() )
       { if( lp.isconvex() ) return '('; else return 0; }
 
     if( urow > b.top() && lrow < b.bottom() && rp.isctip() &&
-        ( bp.ispit() || tp.ispit() || ( bp.islpit() && tp.islpit() ) ) &&
+        ( bp.ispit() || tp.ispit() || ( bp.isltip() && tp.isltip() ) ) &&
         b.escape_right( b.vcenter(), b.hcenter() ) )
       return 'c';
     }
@@ -357,10 +351,8 @@ int Features::test_c() const throw()
   }
 
 
-int Features::test_frst( const Rectangle & charbox ) const throw()
+int Features::test_frst( const Rectangle & charbox ) const
   {
-  const Blob & b = *blob_;
-
   if( bp.minima( b.height() / 4 ) != 1 || tp.minima( b.height() / 2 ) != 1 ||
       bp.minima( b.height() / 2 ) != 1 ) return 0;
   const int noise = ( std::min( b.height(), b.width() ) / 30 ) + 1;
@@ -374,7 +366,7 @@ int Features::test_frst( const Rectangle & charbox ) const throw()
       {
       if( -2 * b_hdiff > b.height() )
         {
-        if( b.height() >= 3 * wp.max() &&
+        if( b.height() >= 3 * wp.max() && !lp.ispit() &&
             ( hbars() == 0 || hbar(0).bottom() < b.vpos( 20 ) ) ) return 'l';
         if( 2 * wp[wp.pos(6)] < b.width() && hbars() >= 1 && hbars() <= 2 &&
             hbar(0).top() >= b.vpos( 15 ) && hbar(0).bottom() < b.vcenter() &&
@@ -429,7 +421,8 @@ int Features::test_frst( const Rectangle & charbox ) const throw()
           }
         }
       if( Ocrad::similar( b.height(), b.width(), 40 ) &&
-          vscan()[lp.pos(15)] == 3 && vscan()[lp.pos(85)] == 3 &&
+          segments_in_row( b.vpos( 15 ) ) == 3 &&
+          segments_in_row( b.vpos( 85 ) ) == 3 &&
           b.seek_right( row - 1, col ) < b.right() && lp.isctip() )
         return 'x';
       }
@@ -451,7 +444,7 @@ int Features::test_frst( const Rectangle & charbox ) const throw()
               tp.minima( noise ) == 2 ||
               ( rp.minima() == 1 && ( b.height() < charbox.height() || tp.iminimum() > tp.pos( 50 ) ) ) )
             { if( b.height() <= 3 * wp.max() ) return 'r'; else return 0; }
-          else if( 3 * b.height() >= 5 * b.width() ) return 'f';
+          else if( 3 * b.height() >= 5 * b.width() && !rp.istip() ) return 'f';
           }
         else
           {
@@ -472,14 +465,12 @@ int Features::test_frst( const Rectangle & charbox ) const throw()
   }
 
 
-int Features::test_G() const throw()
+int Features::test_G() const
   {
-  const Blob & b = *blob_;
-
   if( lp.isconvex() || lp.ispit() )
     {
     int col = 0, row = 0;
-    for( int i = rp.pos( 30 ); i <= rp.pos( 60 ); ++i )
+    for( int i = rp.pos( 60 ); i >= rp.pos( 30 ); --i )
       if( rp[i] > col ) { col = rp[i]; row = i; }
     if( col == 0 ) return 0;
     row += b.top(); col = b.right() - col + 1;
@@ -490,17 +481,19 @@ int Features::test_G() const throw()
     if( row < b.bottom() && b.escape_right( row, col ) &&
         !b.escape_bottom( row, b.hcenter() ) )
       {
+      const int noise = std::max( 2, b.height() / 20 );
       int lrow, urow;
       for( lrow = row - 1 ; lrow > b.top(); --lrow )
         if( b.seek_right( lrow, b.hcenter() ) >= b.right() ) break;
       for( urow = lrow - 1 ; urow > b.top(); --urow )
         if( b.seek_right( urow, b.hcenter() ) < b.right() ) break;
-      lrow += 2; urow -= 1;
+      lrow += noise;
       if( lrow < row && urow > b.top() )
         {
+        urow -= std::min( noise, ( urow - b.top() ) / 2 );
         int uwidth = b.seek_left( urow, b.right() ) - b.seek_right( urow, b.hcenter() );
         int lwidth = b.seek_left( lrow, b.right() ) - b.seek_right( lrow, b.hcenter() );
-        if( lrow - 1 <= b.vcenter() || lwidth > uwidth + 2 )
+        if( lrow - noise <= b.vcenter() || lwidth > uwidth + noise )
           return 'G';
         }
       }
@@ -511,10 +504,8 @@ int Features::test_G() const throw()
 
 // Common feature: U-shaped top of character
 //
-int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const throw()
+int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const
   {
-  const Blob & b = *blob_;
-
   if( tp.minima( b.height() / 5 ) == 2 && tp.minima( b.height() / 4 ) == 2 &&
       tp.minima( b.height() / 2 ) <= 3 && tp.isctip() )
     {
@@ -524,15 +515,20 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const throw()
         ( m5 == 1 ||
           ( m5 == 2 && Ocrad::similar( bp.iminimum(), bp.pos( 50 ), 10 ) ) ) )
       {
+      const int stem = std::min( tp.range() + ( b.height() / 10 ), wp.pos(90) );
+      const bool maybe_Y = ( 5 * tp.range() <= 3 * b.height() ||
+        ( stem <= wp.pos(75) && 5 * wp[stem] <= b.width() ) );
       const int lg = lp.min( lp.pos( 90 ) );
       if( lg > 1 && bp.isvpit() && tp.minima( b.height() / 2 ) == 2 &&
-          lp[lp.pos(75)] <= lg ) return 'v';
+          lp[lp.pos(75)] <= lg &&
+          ( !maybe_Y || 3 * wp[stem] > b.width() || wp[stem] > wp[wp.pos(90)] + 1 ) )
+        return 'v';
       int hdiff;
       if( b.bottom_hook( &hdiff ) )
         {
         if( std::abs( hdiff ) <= b.height() / 8 )
           {
-          if( vscan()[b.height()/4] >= 3 ) return 'v';
+          if( segments_in_row( b.vpos( 30 ) ) >= 3 ) return 'v';
           if( bp.isconvex() )
             { if( 9 * wp[wp.pos(30)] > 10 * wp[wp.pos(50)] &&
                   9 * wp[wp.pos(50)] > 10 * wp[wp.pos(70)] ) return 'v';
@@ -546,13 +542,26 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const throw()
       const int rg2 = rp.max( rp.pos( 70 ), rp.pos( 90 ) );
       const int lc = ( lg + ( 2 * ( lp.limit() - rg ) ) ) / 3;
       const int lc2 = ( lg2 + lp.limit() - rg2 ) / 2;
-      if( bp.ispit() && 7 * tp.range() < 4 * b.height() &&
-          2 * wp[wp.pos(70)] < b.width() &&
-          ( ( Ocrad::similar( lg, rg, 20 ) &&
-            5 * wp[wp.pos(98)] + noise >= 4 * wp[wp.pos(70)] ) ||
-            ( lg > 1 && lg < rg && lc >= lc2 && !rp.increasing() ) ) ) return 'Y';
-      if( b.escape_top( b.vpos( 60 ), b.hcenter() ) && !lp.istip() )
-        return 'u';
+      if( bp.ispit() && maybe_Y )
+        {
+        int row2 = b.top();
+        while( row2 < b.bottom() && segments_in_row( row2 ) != 2 ) ++row2;
+        int row1 = row2 + 1;
+        while( row1 < b.bottom() && segments_in_row( row1 ) != 1 ) ++row1;
+        if( row1 < b.bottom() ) row1 += wp[row1-b.top()] / 4;
+        if( row1 < b.bottom() && wp[row1-b.top()] < b.width() )
+          {
+          const int w1 = wp[row1-b.top()];
+          int row0 = w1 * ( row1 - row2 ) / ( b.width() - w1 ) + row1;
+          if( row0 < b.bottom() && 2 * wp[wp.pos(70)] < b.width() &&
+              ( Ocrad::similar( lg, rg, 20 ) ||
+              ( lg > 1 && lg < rg && lc >= lc2 && !rp.increasing() ) ) )
+            return 'Y';
+          }
+        }
+      if( b.escape_top( b.vpos( 60 ), b.hcenter() ) && !lp.istip() &&
+          ( 4 * b.height() >= 3 * b.width() ||
+            segments_in_col( b.hpos( 75 ) ) <= 2 ) ) return 'u';
       if( lg < rg + 1 && !lp.increasing( lp.pos( 50 ) ) &&
           ( 2 * lg < rg || b.vpos( 90 ) >= charbox.bottom() ) &&
           ( tp.minima( b.height()/2 ) == 1 || lp.imaximum() > b.height()/2 ) )
@@ -563,7 +572,7 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const throw()
           rp.increasing() && tp.minima( b.height() / 2 ) == 2 ) return 'v';
       return 0;
       }
-    if( 2 * b.height() >= b.width() && b.height() >= 10 &&
+    if( 2 * b.height() >= b.width() && b.height() >= 9 &&
         bp.minima() == 2 && bp.isctip() )
       {
       const int th = std::max( b.height() / 4, bp[bp.pos(50)] + noise );
@@ -573,16 +582,20 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const throw()
       if( Ocrad::similar( lg, rg, 80, 2 ) &&
           4 * lg < b.width() && 4 * rg < b.width() )
         {
-        if( lg > 1 && rg > 1 && lp.increasing() && rp.increasing() )
+        if( lg > 1 && rg > 1 && lp.increasing() && rp.increasing() &&
+            5 * tp[tp.pos(50)] > b.height() )
           return 'w';
-        if( hbars() == 1 && 5 * hbar(0).height() < b.height() &&
+        if( hbars() == 1 && 5 * ( hbar(0).height() - 1 ) < b.height() &&
             hbar(0).top() >= b.vpos( 30 ) && hbar(0).bottom() <= b.vpos( 60 ) &&
-            10 * hbar(0).width() > 9 * wp[hbar(0).vcenter()-b.top()] )
+            10 * hbar(0).width() > 9 * wp[hbar(0).vcenter()-b.top()] &&
+            Ocrad::similar( col_segment( hbar(0).vcenter(), hbar(0).hcenter() ).size(),
+                            hbar(0).height(), 30, 2 ) )
           {
-          if( 9 * hbar(0).width() < 10 * wp[wp.pos(50)] ) return 'H';
+          if( 9 * hbar(0).width() <= 10 * wp[wp.pos(50)] ) return 'H';
           return 0;
           }
-        if( vscan()[rp.pos(60)] == 4 || vscan()[rp.pos(70)] == 4 )
+        if( segments_in_row( b.vpos( 60 ) ) == 4 ||
+            segments_in_row( b.vpos( 70 ) ) == 4 )
           {
           if( 2 * tp[tp.pos(50)] > b.height() ) return 'M';
           return 'w';
@@ -592,12 +605,15 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const throw()
             tp.minima( ( 2 * b.height() ) / 5 ) <= 2 && !lp.istpit() &&
             4 * std::abs( rp[rp.pos(20)] - rp[rp.pos(80)] ) <= b.width() )
           {
-          Rectangle r( b.left(), b.top(), b.hcenter(), b.bottom() );
-          Bitmap bm( b, r );
-          int hdiff;
-          if( bm.seek_bottom( bm.top(), bm.right() ) > bm.vcenter() &&
-              bm.bottom_hook( &hdiff ) && -2 * hdiff > bm.height() ) return 'u';
-          else return 'N';
+          const int row = b.top() + tp[tp.pos(50)];
+          if( row > b.vcenter() )
+            {
+            Rectangle r( b.left(), b.top(), b.hcenter(), b.bottom() );
+            Bitmap bm( b, r );
+            int hdiff;
+            if( bm.bottom_hook( &hdiff ) && -2 * hdiff > bm.height() ) return 'u';
+            }
+          if( row > b.vpos( 10 ) || vbars() >= 2 ) return 'N';
           }
         return 0;
         }
@@ -606,7 +622,9 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const throw()
       return 0;
       }
     if( bp.minima() <= 2 && 2 * b.width() > 5 * b.height() ) return '~';
-    if( bp.minima() == 3 ) return 'M';
+    if( bp.minima() == 3 &&
+        ( hbars() == 0 || ( hbars() == 1 && hbar(0).top() >= b.vpos( 20 ) ) ) )
+      return 'M';
     }
   return 0;
   }
@@ -615,13 +633,12 @@ int Features::test_HKMNUuvwYy( const Rectangle & charbox ) const throw()
 // Looks for the nearest frontier in column hcenter(), then tests if
 // gap is open downwards (except for 'x')
 //
-int Features::test_hknwx( const Rectangle & charbox ) const throw()
+int Features::test_hknwx( const Rectangle & charbox ) const
   {
-  const Blob & b = *blob_;
   const int m8 = tp.minima( b.height() / 8 );
 
   if( m8 == 2 && bp.minima( b.height() / 2 ) == 1 &&
-      ( ( lp.istip() && rp.istip() ) ||
+      ( ( lp.isctip() && rp.isctip() ) ||
         ( lp.isconcave() && rp.isconcave() ) ) ) return 'x';
 
   if( b.width() >= b.height() && tp.ispit() &&
@@ -641,6 +658,8 @@ int Features::test_hknwx( const Rectangle & charbox ) const throw()
   if( urow > b.vpos( 20 ) || 3 * tp[tp.pos(60)] > b.height() )
     {
     const int m5 = tp.minima( b.height() / 5 );
+    if( m5 == 3 && segments_in_row( b.vcenter() ) == 2 &&
+        segments_in_row( b.vpos( 80 ) ) == 3 ) return 0;	// merged 'IX'
     if( ( m5 == 2 || m5 == 3 ) && tp.minima() >= 2 &&
         rp[rp.pos(25)] <= b.width() / 4 &&
         ( !lp.istpit() || rp.minima() == 1 ) ) return 'w';
@@ -653,7 +672,9 @@ int Features::test_hknwx( const Rectangle & charbox ) const throw()
       bp.minima( urow + 1 ) == 3 )
     return 'w';
   if( urow <= b.vpos( 20 ) && tp.minima( b.height() / 4 ) == 1 &&
-      ( b.height() < b.width() || Ocrad::similar( b.height(), b.width(), 40 ) ) )
+      Ocrad::similar( b.height(), b.width(), 40 ) &&
+      ( 8 * ( rp[rp.pos(50)] - 1 ) <= b.width() ||
+        tp[tp.pos(99)] > b.height() / 2 ) )
     return 'n';
   return 0;
   }
@@ -662,9 +683,8 @@ int Features::test_hknwx( const Rectangle & charbox ) const throw()
 // Looks for four black sections in column hcenter()  1, then tests if
 // upper gap is open to the right and lower gaps are open to the left
 //
-int Features::test_s_cedilla() const throw()
+int Features::test_s_cedilla() const
   {
-  const Blob & b = *blob_;
   int urow2 = 0, urow3 = 0, urow4 = 0, col, black_section = 0;
 
   for( col = b.hcenter() - 1; col <= b.hcenter() + 1; ++col )
@@ -693,10 +713,8 @@ int Features::test_s_cedilla() const throw()
   }
 
 
-bool Features::test_comma() const throw()
+bool Features::test_comma() const
   {
-  const Blob & b = *blob_;
-
   if( b.holes() || b.height() <= b.width() || b.height() > 3 * b.width() )
     return false;
 
@@ -717,9 +735,8 @@ bool Features::test_comma() const throw()
   }
 
 
-int Features::test_easy( const Rectangle & charbox ) const throw()
+int Features::test_easy( const Rectangle & charbox ) const
   {
-  const Blob & b = *blob_;
   int code = test_solid( charbox );
   if( code ) return code;
 
@@ -737,11 +754,36 @@ int Features::test_easy( const Rectangle & charbox ) const throw()
 
 
 // Recognizes single line, non-rectangular characters without holes.
-// '/<>\^`
+// '/<>C[\^`c
 //
-int Features::test_line( const Rectangle & charbox ) const throw()
+int Features::test_line( const Rectangle & charbox ) const
   {
-  const Blob & b = *blob_;
+  const int vnoise = ( b.height() / 30 ) + 1;
+  const int topmax = b.top() + vnoise;
+  const int botmin = b.bottom() - vnoise;
+  const bool vbar_left = ( vbars() == 1 && vbar(0).width() >= 2 &&
+                           vbar(0).left() <= b.hpos( 10 ) + 1 );
+  if( tp.minima() == 1 && bp.minima() == 1 && rp.istip() )
+    {
+    if( vbar_left && b.height() > 2 * b.width() &&
+        2 * rp[rp.pos(50)] > b.width() )
+      {
+      int row = b.seek_top( b.vcenter(), b.hcenter() );
+      int col = b.seek_right( row, b.hcenter() );
+      if( col < b.right() )
+        {
+        row = b.seek_bottom( b.vcenter(), b.hcenter() );
+        col = b.seek_right( row, b.hcenter() );
+        if( col < b.right() ) return 'C';
+        }
+      }
+    if( hbars() == 2 &&
+        hbar(0).top() <= topmax && 4 * hbar(0).height() <= b.height() &&
+        hbar(1).bottom() >= botmin && 4 * hbar(1).height() <= b.height() )
+      { if( vbar_left && b.height() > 2 * b.width() ) return '[';
+        if( vbar_left || lp.ispit() ) return 'c'; }
+    }
+
   int slope1, slope2;
 
   if( tp.minima() != 1 ) return 0;
@@ -803,9 +845,8 @@ int Features::test_line( const Rectangle & charbox ) const throw()
   }
 
 
-int Features::test_solid( const Rectangle & charbox ) const throw()
+int Features::test_solid( const Rectangle & charbox ) const
   {
-  const Blob & b = *blob_;
   if( b.holes() ) return 0;
 
   if( b.height() >= 5 && b.width() >= 5 )
@@ -820,9 +861,10 @@ int Features::test_solid( const Rectangle & charbox ) const throw()
 
   if( b.width() >= 3 && b.height() >= 3 )
     {
+    const int vnoise = ( b.height() / 100 ) + 1;
     inner_size = ( b.width() - 2 ) * ( b.height() - 2 );
     inner_area = 0;
-    for( int row = b.top() + 1; row < b.bottom(); ++row )
+    for( int row = b.top() + vnoise; row <= b.bottom() - vnoise; ++row )
       {
       int holes = 0;	// FIXME
       for( int col = b.left() + 1; col < b.right(); ++col )
