@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
-    2012, 2013 Antonio Diaz Diaz.
+    Copyright (C) 2003-2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -18,6 +17,7 @@
 
 #include <algorithm>
 #include <cstdio>
+#include <cstdlib>
 #include <vector>
 #include <stdint.h>
 
@@ -39,7 +39,18 @@ int Features::test_49ARegpq( const Rectangle & charbox ) const
   const Bitmap & h = b.hole( 0 );
 
   if( bp.minima( b.height() / 10 + 1 ) == 2 && bp.isctip() && tp.minima() == 1 )
-    { if( tp.isvpit() || rp.decreasing() ) return 'A'; else return 'R'; }
+    {
+    if( tp.isvpit() || rp.decreasing() ||
+        ( rp.decreasing( 1, rp.pos( 20 ) ) && lp.decreasing( 1, lp.pos( 20 ) ) ) )
+      return 'A';
+    if( hbars() == 2 && hbar(1).width() >= b.width() )
+      {
+      const int i = hbar(1).top() - b.top();
+      const int j = hbar(1).bottom() - b.top();
+      if( rp.area( i, j ) <= lp.area( i, j ) ) return 'A';
+      }
+    return 'R';
+    }
 
   int col = h.hcenter();
   int row = b.seek_bottom( h.bottom(), col, false ) + 1;
@@ -70,8 +81,8 @@ int Features::test_49ARegpq( const Rectangle & charbox ) const
         hlp.decreasing() && htp.decreasing() &&
         hwp[hwp.pos(30)] < hwp[hwp.pos(70)] )
       return '4';
-    if( rp.isconvex() && rp.ispit() && rp.minima() == 1 && tp.ispit() &&
-        charbox.bottom() > b.vpos( 80 ) )
+    if( rp.ispit() && rp.minima() == 1 && rp.iminimum() < rp.pos( 70 ) &&
+        tp.ispit() && charbox.bottom() > b.vpos( rp.isconvex() ? 80 : 90 ) )
       return '9';
     int hdiff;
     if( b.bottom_hook( &hdiff ) && hdiff > 0 )
@@ -84,7 +95,8 @@ int Features::test_49ARegpq( const Rectangle & charbox ) const
       }
     if( row > b.vpos( 85 ) && tp.ispit() ) return 'Q';
     int row2 = b.seek_bottom( row, col );
-    if( row2 < b.bottom() && rp.increasing( ( ( row + row2 ) / 2 ) - b.top() ) )
+    if( row2 < b.bottom() &&
+        rp.increasing( ( ( row + ( 2 * row2 ) ) / 3 ) - b.top() ) )
       return 'g';
     if( bp.minima() == 1 )
       {
@@ -136,9 +148,8 @@ int Features::test_4ADQao( const Charset & charset, const Rectangle & charbox )
       if( c == 4 && Ocrad::similar( left_delta, right_delta, 40 ) && lp.ispit() )
         return '@';
       }
-    if( tp.minima() == 1 && bp.istip() && !lp.isconvex() && !rp.isctip( 66 ) )
-      return 'A';
     }
+  if( tp.minima() == 1 && bp.istip() && !rp.isctip( 66 ) ) return 'A';
   if( Ocrad::similar( left_delta, right_delta, 50 ) )
     {
     if( bp.minima() == 1 && rp.isconvex() && b.test_BD() ) return 'D';
@@ -205,7 +216,8 @@ int Features::test_6abd( const Charset & charset ) const
     row2 = b.seek_top( row2, rcol );
     if( row2 <= b.top() ) return 'b';
     const int m = tp.minima( b.height() / 2 );
-    if( m == 1 ) return 's'; else if( m == 2 ) return 'k'; else return 0;
+    if( m == 1 && bp.minima() == 1 ) return 's';
+    if( m == 2 ) return 'k'; else return 0;
     }
 
   if( b.escape_left( row, col ) )
