@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
-    Antonio Diaz Diaz.
+    Copyright (C) 2003-2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -16,18 +15,25 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 /*
-    Return values: 0 for a normal exit, 1 for environmental problems
+    Exit status: 0 for a normal exit, 1 for environmental problems
     (file not found, invalid flags, I/O errors, etc), 2 to indicate a
     corrupt or invalid input file, 3 for an internal consistency error
     (eg, bug) which caused ocrad to panic.
 */
 
 #include <cctype>
+#include <climits>
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
 #include <string>
 #include <vector>
+#include <stdint.h>
+#if defined(__MSVCRT__) || defined(__OS2__) || defined(_MSC_VER)
+#include <fcntl.h>
+#include <unistd.h>
+#include <io.h>
+#endif
 
 #include "arg_parser.h"
 #include "common.h"
@@ -39,10 +45,10 @@
 
 namespace {
 
+const char * const Program_name = "GNU Ocrad";
+const char * const program_name = "ocrad";
+const char * const program_year = "2014";
 const char * invocation_name = 0;
-const char * const Program_name    = "GNU Ocrad";
-const char * const program_name    = "ocrad";
-const char * const program_year    = "2010";
 
 struct Input_control
   {
@@ -51,41 +57,42 @@ struct Input_control
   Rational threshold, ltwh[4];
   bool copy, cut, invert, layout;
 
-  Input_control() throw()
+  Input_control()
     : scale( 0 ), threshold( -1 ),
       copy( false ), cut( false ), invert( false ), layout( false ) {}
 
-  bool parse_cut_rectangle( const char * const s ) throw();
-  bool parse_threshold( const char * const s ) throw();
+  bool parse_cut_rectangle( const char * const s );
+  bool parse_threshold( const char * const s );
   };
 
 
 void show_error( const char * const msg, const int errcode = 0,
-                 const bool help = false ) throw()
+                 const bool help = false )
   {
   if( verbosity >= 0 )
     {
     if( msg && msg[0] )
       {
       std::fprintf( stderr, "%s: %s", program_name, msg );
-      if( errcode > 0 ) std::fprintf( stderr, ": %s", std::strerror( errcode ) );
-      std::fprintf( stderr, "\n" );
+      if( errcode > 0 )
+        std::fprintf( stderr, ": %s.", std::strerror( errcode ) );
+      std::fputs( "\n", stderr );
       }
-    if( help && invocation_name && invocation_name[0] )
-      std::fprintf( stderr, "Try `%s --help' for more information.\n",
+    if( help )
+      std::fprintf( stderr, "Try '%s --help' for more information.\n",
                     invocation_name );
     }
   }
 
 
-bool Input_control::parse_cut_rectangle( const char * const s ) throw()
+bool Input_control::parse_cut_rectangle( const char * const s )
   {
   int c = ltwh[0].parse( s );				// left
-  if( c && s[c] == ',' )
+  if( c && s[c] == ',' && ltwh[0] >= -1 )
     {
     int i = c + 1;
     c = ltwh[1].parse( &s[i] );				// top
-    if( c && s[i+c] == ',' )
+    if( c && s[i+c] == ',' && ltwh[1] >= -1 )
       {
       i += c + 1; c = ltwh[2].parse( &s[i] );		// width
       if( c && s[i+c] == ',' && ltwh[2] > 0 )
@@ -100,7 +107,7 @@ bool Input_control::parse_cut_rectangle( const char * const s ) throw()
   }
 
 
-bool Input_control::parse_threshold( const char * const s ) throw()
+bool Input_control::parse_threshold( const char * const s )
   {
   Rational tmp;
   if( tmp.parse( s ) && tmp >= 0 && tmp <= 1 )
@@ -110,78 +117,105 @@ bool Input_control::parse_threshold( const char * const s ) throw()
   }
 
 
-void show_help() throw()
+void show_help()
   {
-  std::printf( "%s - Optical Character Recognition program.\n", Program_name );
-  std::printf( "Reads pnm file(s), or standard input, and sends text to standard output.\n" );
-  std::printf( "\nUsage: %s [options] [files]\n", invocation_name );
-  std::printf( "\nOptions:\n" );
-  std::printf( "  -h, --help               display this help and exit\n" );
-  std::printf( "  -V, --version            output version information and exit\n" );
-  std::printf( "  -a, --append             append text to output file\n" );
-  std::printf( "  -c, --charset=<name>     try `--charset=help' for a list of names\n" );
-  std::printf( "  -e, --filter=<name>      try `--filter=help' for a list of names\n" );
-  std::printf( "  -f, --force              force overwrite of output file\n" );
-  std::printf( "  -F, --format=<fmt>       output format (byte, utf8)\n" );
-  std::printf( "  -i, --invert             invert image levels (white on black)\n" );
-  std::printf( "  -l, --layout             perform layout analysis\n" );
-  std::printf( "  -o <file>                place the output into <file>\n" );
-  std::printf( "  -q, --quiet              suppress all messages\n" );
-  std::printf( "  -s, --scale=[-]<n>       scale input image by [1/]<n>\n" );
-  std::printf( "  -t, --transform=<name>   try `--transform=help' for a list of names\n" );
-  std::printf( "  -T, --threshold=<n%%>     threshold for binarization (0-100%%)\n" );
-  std::printf( "  -u, --cut=<l,t,w,h>      cut input image by given rectangle\n" );
-  std::printf( "  -v, --verbose            be verbose\n" );
-  std::printf( "  -x <file>                export OCR Results File to <file>\n" );
-  if( verbosity > 0 )
+  std::printf( "GNU Ocrad is an OCR (Optical Character Recognition) program based on a\n"
+               "feature extraction method. It reads images in pbm (bitmap), pgm\n"
+               "(greyscale) or ppm (color) formats and produces text in byte (8-bit) or\n"
+               "UTF-8 formats. The pbm, pgm and ppm formats are collectively known as pnm.\n"
+               "\nOcrad includes a layout analyser able to separate the columns or blocks\n"
+               "of text normally found on printed pages.\n"
+               "\nUsage: %s [options] [files]\n", invocation_name );
+  std::printf( "\nOptions:\n"
+               "  -h, --help               display this help and exit\n"
+               "  -V, --version            output version information and exit\n"
+               "  -a, --append             append text to output file\n"
+               "  -c, --charset=<name>     try '--charset=help' for a list of names\n"
+               "  -e, --filter=<name>      try '--filter=help' for a list of names\n"
+               "  -f, --force              force overwrite of output file\n"
+               "  -F, --format=<fmt>       output format (byte, utf8)\n"
+               "  -i, --invert             invert image levels (white on black)\n"
+               "  -l, --layout             perform layout analysis\n"
+               "  -o, --output=<file>      place the output into <file>\n"
+               "  -q, --quiet              suppress all messages\n"
+               "  -s, --scale=[-]<n>       scale input image by [1/]<n>\n"
+               "  -t, --transform=<name>   try '--transform=help' for a list of names\n"
+               "  -T, --threshold=<n%%>     threshold for binarization (0-100%%)\n"
+               "  -u, --cut=<l,t,w,h>      cut input image by given rectangle\n"
+               "  -v, --verbose            be verbose\n"
+               "  -x, --export=<file>      export results in ORF format to <file>\n" );
+  if( verbosity >= 1 )
     {
-    std::printf( "  -1..6                    pnm output file type (debug)\n" );
-    std::printf( "  -C, --copy               'copy' input to output (debug)\n" );
-    std::printf( "  -D, --debug=<level>      (0-100) output intermediate data (debug)\n" );
+    std::printf( "  -1..6                    pnm output file type (debug)\n"
+                 "  -C, --copy               'copy' input to output (debug)\n"
+                 "  -D, --debug=<level>      (0-100) output intermediate data (debug)\n" );
     }
-  std::printf( "\nReport bugs to bug-ocrad@gnu.org\n" );
-  std::printf( "Ocrad home page: http://www.gnu.org/software/ocrad/ocrad.html\n" );
-  std::printf( "General help using GNU software: http://www.gnu.org/gethelp\n" );
+  std::printf( "If no files are specified, ocrad reads the image from standard input.\n"
+               "If the -o option is not specified, ocrad sends text to standard output.\n"
+               "\nExit status: 0 for a normal exit, 1 for environmental problems (file\n"
+               "not found, invalid flags, I/O errors, etc), 2 to indicate a corrupt or\n"
+               "invalid input file, 3 for an internal consistency error (eg, bug) which\n"
+               "caused ocrad to panic.\n"
+               "\nReport bugs to bug-ocrad@gnu.org\n"
+               "Ocrad home page: http://www.gnu.org/software/ocrad/ocrad.html\n"
+               "General help using GNU software: http://www.gnu.org/gethelp\n" );
   }
 
 
-void show_version() throw()
+void show_version()
   {
-  std::printf( "%s %s\n", Program_name, PROGVERSION );
+  std::printf( "GNU %s %s\n", program_name, PROGVERSION );
   std::printf( "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
-  std::printf( "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n" );
-  std::printf( "This is free software: you are free to change and redistribute it.\n" );
-  std::printf( "There is NO WARRANTY, to the extent permitted by law.\n" );
+  std::printf( "License GPLv2+: GNU GPL version 2 or later <http://gnu.org/licenses/gpl.html>\n"
+               "This is free software: you are free to change and redistribute it.\n"
+               "There is NO WARRANTY, to the extent permitted by law.\n" );
   }
 
 
-const char * my_basename( const char * filename ) throw()
+const char * my_basename( const char * filename )
   {
   const char * c = filename;
-  while( *c ) { if( *c == '/' ) filename = c + 1; ++c; }
+  while( *c ) { if( *c == '/' ) { filename = c + 1; } ++c; }
   return filename;
   }
 
 
 int process_file( FILE * const infile, const char * const infile_name,
                   const Input_control & input_control,
-                  const Control & control ) throw()
+                  const Control & control )
   {
-  if( verbosity > 0 )
-    std::fprintf( stderr, "processing file `%s'\n", infile_name );
+  if( verbosity >= 1 )
+    std::fprintf( stderr, "processing file '%s'\n", infile_name );
   try
     {
     Page_image page_image( infile, input_control.invert );
 
-    if( input_control.cut && !page_image.cut( input_control.ltwh ) )
+    if( input_control.cut )
       {
-      if( verbosity > 0 )
-        std::fprintf( stderr, "file `%s' totally cut away\n", infile_name );
-      return 1;
+      if( page_image.cut( input_control.ltwh ) )
+        {
+        if( verbosity >= 1 )
+          std::fprintf( stderr, "file cut to %dw x %dh\n",
+                        page_image.width(), page_image.height() );
+        }
+      else
+        {
+        if( verbosity >= 1 )
+          std::fprintf( stderr, "file '%s' totally cut away\n", infile_name );
+        return 1;
+        }
       }
-    page_image.threshold( input_control.threshold );
+
     page_image.transform( input_control.transformation );
-    page_image.scale( input_control.scale );
+    page_image.change_scale( input_control.scale );
+    page_image.threshold( input_control.threshold );
+    if( verbosity >= 1 )
+      {
+      const Rational th( page_image.threshold(), page_image.maxval() );
+      std::fprintf( stderr, "maxval = %d, threshold = %d (%s)\n",
+                    page_image.maxval(), page_image.threshold(),
+                    th.to_decimal( 1, -3 ).c_str() );
+      }
 
     if( input_control.copy )
       {
@@ -198,7 +232,7 @@ int process_file( FILE * const infile, const char * const infile_name,
       }
     }
   catch( Page_image::Error e ) { show_error( e.msg ); return 2; }
-  if( verbosity > 0 ) std::fprintf( stderr, "\n" );
+  if( verbosity >= 1 ) std::fputs( "\n", stderr );
   return 0;
   }
 
@@ -215,7 +249,7 @@ int main( const int argc, const char * const argv[] )
   {
   Input_control input_control;
   Control control;
-  const char *outfile_name = 0, *exportfile_name = 0;
+  const char * outfile_name = 0, * exportfile_name = 0;
   bool append = false, force = false;
   invocation_name = argv[0];
 
@@ -237,7 +271,7 @@ int main( const int argc, const char * const argv[] )
     { 'h', "help",      Arg_parser::no  },
     { 'i', "invert",    Arg_parser::no  },
     { 'l', "layout",    Arg_parser::no  },
-    { 'o', 0,           Arg_parser::yes },
+    { 'o', "output",    Arg_parser::yes },
     { 'q', "quiet",     Arg_parser::no  },
     { 's', "scale",     Arg_parser::yes },
     { 't', "transform", Arg_parser::yes },
@@ -245,10 +279,10 @@ int main( const int argc, const char * const argv[] )
     { 'u', "cut",       Arg_parser::yes },
     { 'v', "verbose",   Arg_parser::no  },
     { 'V', "version",   Arg_parser::no  },
-    { 'x', 0,           Arg_parser::yes },
+    { 'x', "export",    Arg_parser::yes },
     {  0 , 0,           Arg_parser::no  } };
 
-  Arg_parser parser( argc, argv, options );
+  const Arg_parser parser( argc, argv, options );
   if( parser.error().size() )				// bad option
     { show_error( parser.error().c_str(), 0, true ); return 1; }
 
@@ -272,8 +306,7 @@ int main( const int argc, const char * const argv[] )
                 break;
       case 'C': input_control.copy = true; break;
       case 'D': control.debug_level = std::strtol( arg, 0, 0 ); break;
-      case 'e': if( !control.filter.set( arg ) )
-                  { control.filter.show_error( program_name, arg ); return 1; }
+      case 'e': if( !control.add_filter( program_name, arg ) ) return 1;
                 break;
       case 'f': force = true; break;
       case 'F': if( !control.set_format( arg ) )
@@ -294,10 +327,15 @@ int main( const int argc, const char * const argv[] )
       case 'v': if( verbosity < 4 ) ++verbosity; break;
       case 'V': show_version(); return 0;
       case 'x': exportfile_name = arg; break;
-      default : Ocrad::internal_error( "uncaught option" );
+      default : Ocrad::internal_error( "uncaught option." );
       }
     } // end process options
 
+#if defined(__MSVCRT__) || defined(__OS2__) || defined(_MSC_VER)
+  setmode( fileno( stdin ), O_BINARY );
+  setmode( fileno( stdout ), O_BINARY );
+#endif
+
   if( outfile_name && std::strcmp( outfile_name, "-" ) != 0 )
     {
     if( append ) control.outfile = std::fopen( outfile_name, "a" );
