@@ -1,9 +1,9 @@
 /*  Ocradlib - Optical Character Recognition library
-    Copyright (C) 2009, 2010, 2011, 2012, 2013 Antonio Diaz Diaz.
+    Copyright (C) 2009-2014 Antonio Diaz Diaz.
 
     This library is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
@@ -12,24 +12,14 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
-    As a special exception, you may use this file as part of a free
-    software library without restriction.  Specifically, if other files
-    instantiate templates or use macros or inline functions from this
-    file, or you compile this file and link it with other files to
-    produce an executable, this file does not by itself cause the
-    resulting executable to be covered by the GNU General Public
-    License.  This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General
-    Public License.
+    along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-const char * const OCRAD_version_string = "0.22";
+const char * const OCRAD_version_string = "0.24";
 
 
 /* OCRAD_Pixmap.data is a pointer to image data formed by "height" rows
@@ -43,7 +33,7 @@ enum OCRAD_Pixmap_Mode { OCRAD_bitmap, OCRAD_greymap, OCRAD_colormap };
 
 struct OCRAD_Pixmap
   {
-  const uint8_t * data;
+  const unsigned char * data;
   int height;
   int width;
   enum OCRAD_Pixmap_Mode mode;
@@ -98,11 +88,11 @@ int OCRAD_result_chars_block( struct OCRAD_Descriptor * const ocrdes,
 
 int OCRAD_result_chars_line( struct OCRAD_Descriptor * const ocrdes,
                              const int blocknum,	// 0..blocks-1
-                             const int linenum );	// 0..lines[block]-1
+                             const int linenum );	// 0..lines(block)-1
 
 const char * OCRAD_result_line( struct OCRAD_Descriptor * const ocrdes,
                                 const int blocknum,	// 0..blocks-1
-                                const int linenum );	// 0..lines[block]-1
+                                const int linenum );	// 0..lines(block)-1
 
 int OCRAD_result_first_character( struct OCRAD_Descriptor * const ocrdes );
 
