@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
-    Antonio Diaz Diaz.
+    Copyright (C) 2003-2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -20,13 +19,16 @@
 #include <cctype>
 #include <climits>
 #include <cstdio>
+#include <cstdlib>
 #include <string>
 #include <vector>
+#include <stdint.h>
 
 #include "ocradlib.h"
 #include "common.h"
 #include "rational.h"
 #include "rectangle.h"
+#include "segment.h"
 #include "mask.h"
 #include "track.h"
 #include "page_image.h"
@@ -36,7 +38,7 @@ namespace {
 
 // binarization by Otsu's method based on maximization of inter-class variance
 //
-int otsu_th( const std::vector< std::vector< unsigned char > > & data,
+int otsu_th( const std::vector< std::vector< uint8_t > > & data,
              const Rectangle & re, const int maxval )
   {
   if( maxval == 1 ) return 0;
@@ -70,15 +72,11 @@ int otsu_th( const std::vector< std::vector< unsigned char > > & data,
       if( bvar > bvar_max ) { bvar_max = bvar; threshold = i; }
       }
 
-  if( verbosity > 0 )
-    std::fprintf( stderr, "maxval = %d, automatic threshold = %d (%s)\n",
-                  maxval, threshold,
-                  Rational( threshold, maxval ).to_decimal( 1, -3 ).c_str() );
   return threshold;
   }
 
 
-int absolute_pos( Rational pos, const int left, const int right ) throw()
+int absolute_pos( Rational pos, const int left, const int right )
   {
   int a;
 
@@ -97,19 +95,99 @@ int absolute_pos( Rational pos, const int left, const int right ) throw()
   }
 
 
-void enlarge_2( std::vector< std::vector< unsigned char > > & data )
+void convol_23( std::vector< std::vector< uint8_t > > & data, const int scale )
   {
-  const int height = data.size(), width = data[0].size();
-  std::vector< std::vector< unsigned char > > new_data( 2 * height );
+  const int height = data.size();
+  const int width = data[0].size();
+  if( height < 3 || width < 3 ) return;
 
-  for( unsigned int row = 0; row < new_data.size(); ++row )
+  std::vector< std::vector< uint8_t > > new_data( height );
+  new_data[0] = data[0];				// copy first row
+  for( int row = 1; row < height - 1; ++row )
+    new_data[row].reserve( width );
+  new_data[height-1] = data[height-1];			// copy last row
+
+  for( int row = 1; row < height - 1; ++row )
+    {
+    const std::vector< uint8_t > & datarow1 = data[row-1];
+    const std::vector< uint8_t > & datarow2 = data[row];
+    const std::vector< uint8_t > & datarow3 = data[row+1];
+    std::vector< uint8_t > & new_datarow = new_data[row];
+    new_datarow.push_back( datarow2[0] );		// copy first col
+    if( scale < 3 )
+      for( int col = 1; col < width - 1; ++col )
+        {
+        int sum = datarow1[col-1] + datarow1[col] + datarow1[col+1] +
+                  datarow2[col-1] + 2 * datarow2[col] + datarow2[col+1] +
+                  datarow3[col-1] + datarow3[col] + datarow3[col+1];
+        new_datarow.push_back( ( sum + 5 ) / 10 );
+        }
+    else
+      for( int col = 1; col < width - 1; ++col )
+        {
+        int sum = datarow1[col-1] + datarow1[col] + datarow1[col+1] +
+                  datarow2[col-1] + datarow2[col] + datarow2[col+1] +
+                  datarow3[col-1] + datarow3[col] + datarow3[col+1];
+        new_datarow.push_back( ( 2 * sum + 9 ) / 18 );
+        }
+    new_datarow.push_back( datarow2[width-1] );		// copy last col
+    }
+  data.swap( new_data );
+  }
+
+
+void convol_n( std::vector< std::vector< uint8_t > > & data, const int scale )
+  {
+  const int radius = scale / 2;		// this is really radius - 0.5
+  const int min_size = 2 * radius + 1;
+  const int area = min_size * min_size;
+  const int height = data.size();
+  const int width = data[0].size();
+  if( radius < 1 || height < min_size || width < min_size ) return;
+
+  std::vector< std::vector< uint8_t > > new_data( height );
+  for( int row = 0; row < radius; ++row )
+    new_data[row] = data[row];				// copy first rows
+  for( int row = radius; row < height - radius; ++row )
+    new_data[row].reserve( width );
+  for( int row = height - radius; row < height; ++row )
+    new_data[row] = data[row];				// copy last rows
+
+  for( int row = radius; row < height - radius; ++row )
+    {
+    const std::vector< uint8_t > & datarow = data[row];
+    std::vector< uint8_t > & new_datarow = new_data[row];
+    for( int col = 0; col < radius; ++col )
+      new_datarow.push_back( datarow[col] );		// copy first cols
+    for( int col = radius; col < width - radius; ++col )
+      {
+      int sum = 0;
+      for( int r = -radius; r < radius; ++r )
+        for( int c = -radius; c < radius; ++c )
+          sum += data[row+r][col+c];
+      new_datarow.push_back( ( 2 * sum + area ) / ( 2 * area ) );
+      }
+    for( int col = width - radius; col < width; ++col )
+      new_datarow.push_back( datarow[col] );		// copy last cols
+    }
+  data.swap( new_data );
+  }
+
+
+void enlarge_2b( std::vector< std::vector< uint8_t > > & data )
+  {
+  const int height = data.size();
+  const int width = data[0].size();
+  std::vector< std::vector< uint8_t > > new_data( 2 * height );
+
+  for( unsigned row = 0; row < new_data.size(); ++row )
     new_data[row].resize( 2 * width, 1 );
 
   for( int row = 0; row < height; ++row )
     {
-    const std::vector< unsigned char > & datarow = data[row];
-    std::vector< unsigned char > & new_datarow0 = new_data[2*row];
-    std::vector< unsigned char > & new_datarow1 = new_data[2*row+1];
+    const std::vector< uint8_t > & datarow = data[row];
+    std::vector< uint8_t > & new_datarow0 = new_data[2*row];
+    std::vector< uint8_t > & new_datarow1 = new_data[2*row+1];
     for( int col = 0; col < width; ++col )
       {
       if( datarow[col] == 0 )
@@ -134,21 +212,22 @@ void enlarge_2( std::vector< std::vector< unsigned char > > & data )
   }
 
 
-void enlarge_3( std::vector< std::vector< unsigned char > > & data )
+void enlarge_3b( std::vector< std::vector< uint8_t > > & data )
   {
-  const int height = data.size(), width = data[0].size();
-  std::vector< std::vector< unsigned char > > new_data( 3 * height );
+  const int height = data.size();
+  const int width = data[0].size();
+  std::vector< std::vector< uint8_t > > new_data( 3 * height );
 
-  for( unsigned int row = 0; row < new_data.size(); ++row )
+  for( unsigned row = 0; row < new_data.size(); ++row )
     new_data[row].resize( 3 * width, 1 );
 
   for( int row = 0; row < height; ++row )
     {
     const int row3 = 3 * row;
-    const std::vector< unsigned char > & datarow = data[row];
-    std::vector< unsigned char > & new_datarow0 = new_data[row3];
-    std::vector< unsigned char > & new_datarow1 = new_data[row3+1];
-    std::vector< unsigned char > & new_datarow2 = new_data[row3+2];
+    const std::vector< uint8_t > & datarow = data[row];
+    std::vector< uint8_t > & new_datarow0 = new_data[row3];
+    std::vector< uint8_t > & new_datarow1 = new_data[row3+1];
+    std::vector< uint8_t > & new_datarow2 = new_data[row3+2];
     for( int col = 0; col < width; ++col )
       {
       const int col3 = 3 * col;
@@ -183,20 +262,21 @@ void enlarge_3( std::vector< std::vector< unsigned char > > & data )
   }
 
 
-void enlarge_n( std::vector< std::vector< unsigned char > > & data, const int n )
+void enlarge_n( std::vector< std::vector< uint8_t > > & data, const int n )
   {
   if( n < 2 ) return;
-  const int height = data.size(), width = data[0].size();
-  std::vector< std::vector< unsigned char > > new_data;
+  const int height = data.size();
+  const int width = data[0].size();
+  std::vector< std::vector< uint8_t > > new_data;
   new_data.reserve( n * height );
 
   for( int row = 0; row < height; ++row )
     {
-    const std::vector< unsigned char > & datarow = data[row];
-    new_data.push_back( std::vector< unsigned char >() );
+    const std::vector< uint8_t > & datarow = data[row];
+    new_data.push_back( std::vector< uint8_t >() );
     for( int col = 0; col < width; ++col )
       {
-      const unsigned char d = datarow[col];
+      const uint8_t d = datarow[col];
       for( int i = 0; i < n; ++i ) new_data.back().push_back( d );
       }
     for( int i = 1; i < n; ++i ) new_data.push_back( new_data.back() );
@@ -205,7 +285,7 @@ void enlarge_n( std::vector< std::vector< unsigned char > > & data, const int n
   }
 
 
-void mirror_left_right( std::vector< std::vector< unsigned char > > & data ) throw()
+void mirror_left_right( std::vector< std::vector< uint8_t > > & data )
   {
   const int height = data.size();
   for( int row = 0; row < height; ++row )
@@ -213,14 +293,15 @@ void mirror_left_right( std::vector< std::vector< unsigned char > > & data ) thr
   }
 
 
-void mirror_top_bottom( std::vector< std::vector< unsigned char > > & data ) throw()
+void mirror_top_bottom( std::vector< std::vector< uint8_t > > & data )
   {
-  for( int u = 0, d = data.size() - 1; u < d; ++u, --d )
+  const int height = data.size();
+  for( int u = 0, d = height - 1; u < d; ++u, --d )
     data[u].swap( data[d] );
   }
 
 
-void mirror_diagonal( std::vector< std::vector< unsigned char > > & data,
+void mirror_diagonal( std::vector< std::vector< uint8_t > > & data,
                       Rectangle & re )
   {
   const int size = std::max( re.height(), re.width() );
@@ -237,10 +318,10 @@ void mirror_diagonal( std::vector< std::vector< unsigned char > > & data,
 
   for( int row = 0; row < size; ++row )
     {
-    std::vector< unsigned char > & datarow = data[row];
+    std::vector< uint8_t > & datarow = data[row];
     for( int col = 0; col < row; ++col )
       {
-      unsigned char tmp = datarow[col];
+      uint8_t tmp = datarow[col];
       datarow[col] = data[col][row]; data[col][row] = tmp;
       }
     }
@@ -262,7 +343,7 @@ Page_image::Page_image( const OCRAD_Pixmap & image, const bool invert )
   : Rectangle( 0, 0, image.width - 1, image.height - 1 )
   {
   data.resize( height() );
-  for( unsigned int row = 0; row < data.size(); ++row )
+  for( unsigned row = 0; row < data.size(); ++row )
     data[row].reserve( width() );
   const int rows = height(), cols = width();
 
@@ -295,10 +376,10 @@ Page_image::Page_image( const OCRAD_Pixmap & image, const bool invert )
       for( int i = 0, row = 0; row < rows; ++row )
         for( int col = 0; col < cols; ++col, i += 3 )
           {
-          const unsigned char r = image.data[i];	// Red value
-          const unsigned char g = image.data[i+1];	// Green value
-          const unsigned char b = image.data[i+2];	// Blue value
-          unsigned char val;
+          const uint8_t r = image.data[i];		// Red value
+          const uint8_t g = image.data[i+1];		// Green value
+          const uint8_t b = image.data[i+2];		// Blue value
+          uint8_t val;
           if( !invert ) val = std::min( r, std::min( g, b ) );
           else val = maxval_ - std::max( r, std::max( g, b ) );
           data[row].push_back( val );
@@ -314,7 +395,7 @@ Page_image::Page_image( const Page_image & source, const int scale )
   : Rectangle( source ), maxval_( source.maxval_ ), threshold_( source.threshold_ )
   {
   if( scale < 2 || scale > source.width() || scale > source.height() )
-    Ocrad::internal_error( "bad parameter building a reduced Page_image" );
+    Ocrad::internal_error( "bad parameter building a reduced Page_image." );
 
   const int scale2 = scale * scale;
   Rectangle::height( source.height() / scale );
@@ -325,14 +406,14 @@ Page_image::Page_image( const Page_image & source, const int scale )
     {
     const int srow = ( row * scale ) + scale;
     data[row].reserve( width() );
-    std::vector< unsigned char > & datarow = data[row];
+    std::vector< uint8_t > & datarow = data[row];
     for( int col = 0; col < width(); ++col )
       {
       const int scol = ( col * scale ) + scale;
       int sum = 0;
       for( int i = srow - scale; i < srow; ++i )
         {
-        const std::vector< unsigned char > & sdatarow = source.data[i];
+        const std::vector< uint8_t > & sdatarow = source.data[i];
         for( int j = scol - scale; j < scol; ++j )
           sum += sdatarow[j];
         }
@@ -345,13 +426,9 @@ Page_image::Page_image( const Page_image & source, const int scale )
 void Page_image::threshold( const Rational & th )
   {
   if( th >= 0 && th <= 1 )
-    {
     threshold_ = ( th * maxval_ ).trunc();
-    if( verbosity > 0 )
-      std::fprintf( stderr, "maxval = %d, manual threshold = %d (%s)\n",
-                    maxval_, threshold_, th.to_decimal( 1, -3 ).c_str() );
-    }
-  else threshold_ = otsu_th( data, *this, maxval_ );
+  else
+    threshold_ = otsu_th( data, *this, maxval_ );
   }
 
 
@@ -400,13 +477,11 @@ bool Page_image::cut( const Rational ltwh[4] )
   Rectangle::right( data[0].size() - 1 );
   Rectangle::bottom( data.size() - 1 );
 
-  if( verbosity > 0 )
-    std::fprintf( stderr, "file cut to %dw x %dh\n", width(), height() );
   return true;
   }
 
 
-void Page_image::draw_mask( const Mask & m ) throw()
+void Page_image::draw_mask( const Mask & m )
   {
   const int t = std::max( top(), m.top() );
   const int b = std::min( bottom(), m.bottom() );
@@ -443,7 +518,7 @@ void Page_image::draw_mask( const Mask & m ) throw()
   }
 
 
-void Page_image::draw_rectangle( const Rectangle & re ) throw()
+void Page_image::draw_rectangle( const Rectangle & re )
   {
   const int l = std::max( left(), re.left() );
   const int t = std::max( top(), re.top() );
@@ -460,7 +535,7 @@ void Page_image::draw_rectangle( const Rectangle & re ) throw()
   }
 
 
-void Page_image::draw_track( const Track & tr ) throw()
+void Page_image::draw_track( const Track & tr )
   {
   int l = std::max( left(), tr.left() );
   int r = std::min( right(), tr.right() );
@@ -480,20 +555,25 @@ void Page_image::draw_track( const Track & tr ) throw()
   }
 
 
-bool Page_image::scale( int n )
+bool Page_image::change_scale( int n )
   {
   if( n <= -2 )
     { Page_image reduced( *this, -n ); *this = reduced; return true; }
   if( n >= 2 )
     {
-    if( (long long)width() * height() * n > (long long)INT_MAX )
-      throw Error( "scale factor too big. `int' will overflow." );
+    if( INT_MAX / n < width() * height() )
+      throw Error( "scale factor too big. 'int' will overflow." );
     if( maxval_ == 1 )
       {
-      if( n && ( n % 2 ) == 0 ) { enlarge_2( data ); n /= 2; }
-      else if( n && ( n % 3 ) == 0 ) { enlarge_3( data ); n /= 3; }
+      if( n && ( n % 2 ) == 0 ) { enlarge_2b( data ); n /= 2; }
+      else if( n && ( n % 3 ) == 0 ) { enlarge_3b( data ); n /= 3; }
+      }
+    if( n >= 2 )
+      {
+      enlarge_n( data, n );
+      if( maxval_ > 1 )
+        { if( n <= 3 ) convol_23( data, n ); else convol_n( data, n ); }
       }
-    if( n > 1 ) enlarge_n( data, n );
     Rectangle::height( data.size() );
     Rectangle::width( data[0].size() );
     return true;
