@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
-    Antonio Diaz Diaz.
+    Copyright (C) 2003-2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -27,12 +26,19 @@ public:
   struct Error
     {
     const char * const msg;
-    Error( const char * const s ) : msg( s ) {}
+    explicit Error( const char * const s ) : msg( s ) {}
     };
 
+  using Rectangle::left;
+  using Rectangle::top;
+  using Rectangle::right;
+  using Rectangle::bottom;
+  using Rectangle::height;
+  using Rectangle::width;
+
 private:
-  std::vector< std::vector< unsigned char > > data;	// 256 level greymap
-  unsigned char maxval_, threshold_;		// x > threshold == white
+  std::vector< std::vector< uint8_t > > data;	// 256 level greymap
+  uint8_t maxval_, threshold_;			// x > threshold == white
 
   void read_p1( FILE * const f, const bool invert );
   void read_p4( FILE * const f, const bool invert );
@@ -41,6 +47,13 @@ private:
   void read_p3( FILE * const f, const bool invert );
   void read_p6( FILE * const f, const bool invert );
 
+  void left  ( int );		// resize functions declared as private
+  void top   ( int );
+  void right ( int );
+  void bottom( int );
+  void height( int );
+  void width ( int );
+
 public:
   // Creates a Page_image from a pbm, pgm or ppm file
   Page_image( FILE * const f, const bool invert );
@@ -51,36 +64,23 @@ public:
   // Creates a reduced Page_image
   Page_image( const Page_image & source, const int scale );
 
-  using Rectangle::left;
-  using Rectangle::top;
-  using Rectangle::right;
-  using Rectangle::bottom;
-  using Rectangle::height;
-  using Rectangle::width;
-  void left  ( int ) { throw Error( "Page_image resize not allowed." ); }
-  void top   ( int ) { left( 0 ); }
-  void right ( int ) { left( 0 ); }
-  void bottom( int ) { left( 0 ); }
-  void height( int ) { left( 0 ); }
-  void width ( int ) { left( 0 ); }
-
-  bool get_bit( const int row, const int col ) const throw()
+  bool get_bit( const int row, const int col ) const
     { return data[row-top()][col-left()] <= threshold_; }
-  bool get_bit( const int row, const int col, const unsigned char th ) const throw()
+  bool get_bit( const int row, const int col, const uint8_t th ) const
     { return data[row-top()][col-left()] <= th; }
-  void set_bit( const int row, const int col, const bool bit ) throw()
+  void set_bit( const int row, const int col, const bool bit )
     { data[row-top()][col-left()] = ( bit ? 0 : maxval_ ); }
 
-  unsigned char maxval() const throw() { return maxval_; }
-  unsigned char threshold() const throw() { return threshold_; }
+  uint8_t maxval() const { return maxval_; }
+  uint8_t threshold() const { return threshold_; }
   void threshold( const Rational & th );	// 0 <= th <= 1, else auto
   void threshold( const int th );		// 0 <= th <= 255, else auto
 
-  bool cut( const Rational ltwh[4] ) throw();
-  void draw_mask( const Mask & m ) throw();
-  void draw_rectangle( const Rectangle & re ) throw();
-  void draw_track( const Track & tr ) throw();
-  bool save( FILE * const f, const char filetype ) const throw();
-  bool scale( int n );
-  void transform( const Transformation & t ) throw();
+  bool cut( const Rational ltwh[4] );
+  void draw_mask( const Mask & m );
+  void draw_rectangle( const Rectangle & re );
+  void draw_track( const Track & tr );
+  bool save( FILE * const f, const char filetype ) const;
+  bool change_scale( int n );
+  void transform( const Transformation & t );
   };
