@@ -1,6 +1,6 @@
 #! /bin/sh
 # check script for GNU Ocrad - Optical Character Recognition program
-# Copyright (C) 2009, 2010, 2011, 2012, 2013 Antonio Diaz Diaz.
+# Copyright (C) 2009-2014 Antonio Diaz Diaz.
 #
 # This script is free software: you have unlimited permission
 # to copy, distribute and modify it.
@@ -10,10 +10,10 @@ export LC_ALL
 objdir=`pwd`
 testdir=`cd "$1" ; pwd`
 OCRAD="${objdir}"/ocrad
-OCRCHECK="${objdir}"/ocrcheck
+OCRADCHECK="${objdir}"/ocradcheck
 framework_failure() { echo "failure in testing framework" ; exit 1 ; }
 
-if [ ! -x "${OCRAD}" ] ; then
+if [ ! -f "${OCRAD}" ] || [ ! -x "${OCRAD}" ] ; then
 	echo "${OCRAD}: cannot execute"
 	exit 1
 fi
@@ -30,13 +30,13 @@ fail=0
 printf "testing ocrad-%s..." "$2"
 
 "${OCRAD}" -q -T-0.1 ${in} > /dev/null
-if [ $? != 1 ] ; then fail=1 ; printf - ; else printf . ; fi
+if [ $? = 1 ] ; then printf . ; else printf - ; fail=1 ; fi
 "${OCRAD}" -q -T 1.1 ${in} > /dev/null
-if [ $? != 1 ] ; then fail=1 ; printf - ; else printf . ; fi
+if [ $? = 1 ] ; then printf . ; else printf - ; fail=1 ; fi
 "${OCRAD}" -q -u -2,-1,1,1 ${in} > /dev/null
-if [ $? != 1 ] ; then fail=1 ; printf - ; else printf . ; fi
+if [ $? = 1 ] ; then printf . ; else printf - ; fail=1 ; fi
 "${OCRAD}" -q -u 1,1,1,1 ${in} > /dev/null
-if [ $? != 1 ] ; then fail=1 ; printf - ; else printf . ; fi
+if [ $? = 1 ] ; then printf . ; else printf - ; fail=1 ; fi
 
 "${OCRAD}" -q ${in} > out || fail=1
 cmp ${txt} out || fail=1
@@ -115,10 +115,10 @@ expected_chars="+*/#$&()?[\\]^{|}<>@~"
 produced_chars=
 test_chars
 
-"${OCRCHECK}" ${in} > out || fail=1
+"${OCRADCHECK}" ${in} > out || fail=1
 cmp ${txt} out || fail=1
 printf .
-"${OCRCHECK}" ${in} --utf8 > out || fail=1
+"${OCRADCHECK}" ${in} --utf8 > out || fail=1
 cmp ${utxt} out || fail=1
 printf .
 
