@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
-    Antonio Diaz Diaz.
+    Copyright (C) 2003-2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -18,11 +17,14 @@
 
 #include <algorithm>
 #include <cctype>
+#include <climits>
 #include <cstdio>
 #include <string>
 #include <vector>
+#include <stdint.h>
 
 #include "common.h"
+#include "histogram.h"
 #include "rational.h"
 #include "rectangle.h"
 #include "track.h"
@@ -38,13 +40,14 @@ namespace {
 
 // Return the character position >= first preceding a big gap or eol.
 //
-int find_big_gap( const Textline & line, const int first, const int space_width_limit ) throw()
+int find_big_gap( const Textline & line, const int first,
+                  const int space_width_limit )
   {
   int i = first;
   while( i + 1 < line.characters() )
     {
-    Character & c1 = line.character( i );
-    Character & c2 = line.character( i + 1 );
+    const Character & c1 = line.character( i );
+    const Character & c2 = line.character( i + 1 );
     const int gap = c2.left() - c1.right() - 1;
     if( gap > space_width_limit ) break; else ++i;
     }
@@ -54,66 +57,63 @@ int find_big_gap( const Textline & line, const int first, const int space_width_
 } // end namespace
 
 
-Textline::Textline( const Textline & tl ) throw()
+Textline::Textline( const Textline & tl )
   : Track( tl ), big_initials_( tl.big_initials_ )
   {
   cpv.reserve( tl.cpv.size() );
-  for( unsigned int i = 0; i < tl.cpv.size(); ++i )
+  for( unsigned i = 0; i < tl.cpv.size(); ++i )
     cpv.push_back( new Character( *tl.cpv[i] ) );
   }
 
 
-Textline & Textline::operator=( const Textline & tl ) throw()
+Textline & Textline::operator=( const Textline & tl )
   {
   if( this != &tl )
     {
     Track::operator=( tl );
     big_initials_ = tl.big_initials_;
-    for( unsigned int i = 0; i < cpv.size(); ++i ) delete cpv[i];
+    for( unsigned i = 0; i < cpv.size(); ++i ) delete cpv[i];
     cpv.clear();
     cpv.reserve( tl.cpv.size() );
-    for( unsigned int i = 0; i < tl.cpv.size(); ++i )
+    for( unsigned i = 0; i < tl.cpv.size(); ++i )
       cpv.push_back( new Character( *tl.cpv[i] ) );
     }
   return *this;
   }
 
 
-Textline::~Textline() throw()
+Textline::~Textline()
   {
-  for( unsigned int i = 0; i < cpv.size(); ++i ) delete cpv[i];
+  for( unsigned i = 0; i < cpv.size(); ++i ) delete cpv[i];
   }
 
 
-void Textline::set_track() throw()
+void Textline::set_track()
   {
   std::vector< Rectangle > rv;
-  for( unsigned int i = big_initials_; i < cpv.size(); ++i )
+  for( unsigned i = big_initials_; i < cpv.size(); ++i )
     if( !cpv[i]->maybe(' ') ) rv.push_back( *cpv[i] );
   Track::set_track( rv );
   }
 
 
-void Textline::verify_big_initials() throw()
+void Textline::verify_big_initials()
   {
-  if( big_initials_ )
-    {
-    const int mh = mean_height();
-    while( big_initials_ && cpv[big_initials_-1]->height() <= 2 * mh )
-      --big_initials_;
-    }
+  while( big_initials_ > 0 &&
+         cpv[big_initials_-1]->height() <= 2 * mean_height() )
+    --big_initials_;
   }
 
 
-Character & Textline::character( const int i ) const throw()
+Character & Textline::character( const int i ) const
   {
   if( i < 0 || i >= characters() )
-    Ocrad::internal_error( "character, index out of bounds" );
+    Ocrad::internal_error( "character, index out of bounds." );
   return *cpv[i];
   }
 
 
-Character * Textline::character_at( const int col ) const throw()
+Character * Textline::character_at( const int col ) const
   {
   for( int i = 0; i < characters(); ++i )
     if( cpv[i]->h_includes( col ) ) return cpv[i];
@@ -121,22 +121,22 @@ Character * Textline::character_at( const int col ) const throw()
   }
 
 
-Rectangle Textline::charbox( const Character & c ) const throw()
+Rectangle Textline::charbox( const Character & c ) const
   {
   return Rectangle( c.left(), top( c.hcenter() ), c.right(), bottom( c.hcenter() ) );
   }
 
 
-void Textline::delete_character( const int i ) throw()
+void Textline::delete_character( const int i )
   {
   if( i < 0 || i >= characters() )
-    Ocrad::internal_error( "delete_character, index out of bounds" );
+    Ocrad::internal_error( "delete_character, index out of bounds." );
   if( i < big_initials_ ) --big_initials_;
   delete cpv[i]; cpv.erase( cpv.begin() + i );
   }
 
 
-int Textline::shift_characterp( Character * p, const bool big ) throw()
+int Textline::shift_characterp( Character * const p, const bool big )
   {
   int i = characters();
 
@@ -148,12 +148,12 @@ int Textline::shift_characterp( Character * p, const bool big ) throw()
   }
 
 
-bool Textline::insert_space( const int i, const bool tab ) throw()
+bool Textline::insert_space( const int i, const bool tab )
   {
   if( i <= 0 || i >= characters() )
-    Ocrad::internal_error( "insert_space, index out of bounds" );
+    Ocrad::internal_error( "insert_space, index out of bounds." );
   if( !height() )
-    Ocrad::internal_error( "insert_space, track not set yet" );
+    Ocrad::internal_error( "insert_space, track not set yet." );
   Character & c1 = *cpv[i-1];
   Character & c2 = *cpv[i];
   int l = c1.right() + 1;
@@ -162,7 +162,7 @@ bool Textline::insert_space( const int i, const bool tab ) throw()
   int t = top( ( l + r ) / 2 );
   int b = bottom( ( l + r ) / 2 );
   Rectangle re( l, t, r, b );
-  Character * p = new Character( re, ' ', tab ? 1 : 0 );
+  Character * const p = new Character( re, ' ', tab ? 1 : 0 );
   if( tab ) p->add_guess( '\t', 0 );
   cpv.insert( cpv.begin() + i, p );
   return true;
@@ -171,7 +171,7 @@ bool Textline::insert_space( const int i, const bool tab ) throw()
 
 // Insert spaces between characters.
 //
-void Textline::insert_spaces() throw()
+void Textline::insert_spaces()
   {
   const Rational mw = mean_width();
   if( mw < 2 ) return;
@@ -187,8 +187,8 @@ void Textline::insert_spaces() throw()
       int spaces = 0, nospaces = 0, spsum = 0, nospsum = 0;
       for( int i = first ; i < last; ++i )
         {
-        Character & c1 = character( i );
-        Character & c2 = character( i + 1 );
+        const Character & c1 = character( i );
+        const Character & c2 = character( i + 1 );
         const int gap = c2.left() - c1.right() - 1;
         if( gap >= mwt || gap > 3 * mg ||
             ( 5 * gap > 2 * mw && gap > 2 * mg ) ||
@@ -202,8 +202,8 @@ void Textline::insert_spaces() throw()
         const Rational th = ( Rational( 3 * spsum, spaces ) + Rational( nospsum, nospaces ) ) / 4;
         for( int i = first ; i < last; ++i )
           {
-          Character & c1 = character( i );
-          Character & c2 = character( i + 1 );
+          const Character & c1 = character( i );
+          const Character & c2 = character( i + 1 );
           const int gap = c2.left() - c1.right() - 1;
           if( gap > th && insert_space( i + 1 ) ) { ++i; ++last; }
           }
@@ -215,7 +215,7 @@ void Textline::insert_spaces() throw()
   }
 
 
-void Textline::join( Textline & tl ) throw()
+void Textline::join( Textline & tl )
   {
   for( int i = 0; i < tl.characters(); ++i )
     shift_characterp( tl.cpv[i], i < tl.big_initials_ );
@@ -223,7 +223,7 @@ void Textline::join( Textline & tl ) throw()
   }
 
 
-int Textline::mean_height() const throw()
+int Textline::mean_height() const
   {
   int c = 0, sum = 0;
 
@@ -234,7 +234,7 @@ int Textline::mean_height() const throw()
   }
 
 
-Rational Textline::mean_width() const throw()
+Rational Textline::mean_width() const
   {
   int c = 0, sum = 0;
 
@@ -246,7 +246,7 @@ Rational Textline::mean_width() const throw()
   }
 
 
-Rational Textline::mean_gap_width( const int first, int last ) const throw()
+Rational Textline::mean_gap_width( const int first, int last ) const
   {
   if( last < 0 ) last = characters() - 1;
   int sum = 0;
@@ -259,7 +259,7 @@ Rational Textline::mean_gap_width( const int first, int last ) const throw()
   }
 
 
-int Textline::mean_hcenter() const throw()
+int Textline::mean_hcenter() const
   {
   int c = 0, sum = 0;
 
@@ -270,7 +270,7 @@ int Textline::mean_hcenter() const throw()
   }
 
 
-int Textline::mean_vcenter() const throw()
+int Textline::mean_vcenter() const
   {
   int c = 0, sum = 0;
 
@@ -281,23 +281,29 @@ int Textline::mean_vcenter() const throw()
   }
 
 
-void Textline::print( const Control & control ) const throw()
+void Textline::print( const Control & control ) const
   {
   for( int i = 0; i < characters(); ++i ) character( i ).print( control );
   std::fputs( "\n", control.outfile );
   }
 
 
-void Textline::dprint( const Control & control, bool graph, bool recursive )
-								const throw()
+void Textline::dprint( const Control & control, const bool graph,
+                       const bool recursive ) const
   {
   if( graph || recursive )
-    std::fprintf( control.outfile, "mean height = %d, track segments = %d\n",
-                  mean_height(), segments() );
+    {
+    Histogram hist;
+    for( int i = 0; i < characters(); ++i )
+      if( !character(i).maybe(' ') )
+        hist.add_sample( character(i).height() );
+    std::fprintf( control.outfile, "mean height = %d, median height = %d, track segments = %d\n",
+                  mean_height(), hist.median(), segments() );
+    }
 
   for( int i = 0; i < characters(); ++i )
     {
-    Character & c = character( i );
+    const Character & c = character( i );
     if( i < big_initials_ ) c.dprint( control, c, graph, recursive );
     else c.dprint( control, charbox( c ), graph, recursive );
     }
@@ -305,21 +311,21 @@ void Textline::dprint( const Control & control, bool graph, bool recursive )
   }
 
 
-void Textline::xprint( const Control & control ) const throw()
+void Textline::xprint( const Control & control ) const
   {
   for( int i = 0; i < characters(); ++i )
     character( i ).xprint( control );
   }
 
 
-void Textline::cmark( Page_image & page_image ) const throw()
+void Textline::cmark( Page_image & page_image ) const
   {
   for( int i = 0; i < characters(); ++i )
     page_image.draw_rectangle( character( i ) );
   }
 
 
-void Textline::recognize1( const Charset & charset ) const throw()
+void Textline::recognize1( const Charset & charset ) const
   {
   for( int i = 0; i < characters(); ++i )
     {
@@ -339,20 +345,31 @@ void Textline::recognize1( const Charset & charset ) const throw()
   }
 
 
-void Textline::apply_filter( const Filter & filter ) throw()
+void Textline::apply_filter( const Filter::Type filter )
   {
-  bool flag = false;
-  for( int i = 0; i < characters(); )
+  bool modified = false;
+  for( int i = characters() - 1; i >= 0; --i )
     {
     Character & c = character( i );
-    const int guesses = c.guesses();
+    if( !c.guesses() ) continue;
     c.apply_filter( filter );
-    if( guesses && !c.guesses() ) { delete_character( i ); flag = true; }
-    else ++i;
+    if( !c.guesses() ) { delete_character( i ); modified = true; }
     }
-  if( flag )
+  if( filter == Filter::same_height )
+    {
+    Histogram hist;
+    for( int i = 0; i < characters(); ++i )
+      if( !character(i).maybe(' ') )
+        hist.add_sample( character(i).height() );
+    const int median_height = hist.median();
+    for( int i = characters() - 1; i >= 0; --i )
+      if( !character(i).maybe(' ') &&
+          !Ocrad::similar( character(i).height(), median_height, 13, 2 ) )
+        { delete_character( i ); modified = true; }
+    }
+  if( modified )		// remove leadind/trailing/duplicate spaces
     for( int i = characters() - 1; i >= 0; --i )
       if( character(i).maybe(' ') &&
-          ( i >= characters() - 1 || ( i > 0 && character(i-1).maybe(' ') ) ) )
+          ( i == 0 || i == characters() - 1 || character(i-1).maybe(' ') ) )
         delete_character( i );
   }
