@@ -1,10 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
-    2012, 2013 Antonio Diaz Diaz.
+    Copyright (C) 2003-2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -18,6 +17,7 @@
 
 #include <algorithm>
 #include <cstdio>
+#include <cstdlib>
 #include <vector>
 
 #include "common.h"
@@ -143,7 +143,7 @@ Vrhomboid::Vrhomboid( const int l, const int lc, const int r, const int rc,
     if( verbosity >= 0 )
       std::fprintf( stderr, "l = %d, lc = %d, r = %d, rc = %d, h = %d\n",
                     l, lc, r, rc, h );
-    error( "bad parameter building a Vrhomboid" );
+    error( "bad parameter building a Vrhomboid." );
     }
   left_ = l; lvcenter_ = lc; right_ = r; rvcenter_ = rc; height_ = h;
   }
@@ -151,21 +151,21 @@ Vrhomboid::Vrhomboid( const int l, const int lc, const int r, const int rc,
 
 void Vrhomboid::left( const int l )
   {
-  if( l > right_ ) error( "left, bad parameter resizing a Vrhomboid" );
+  if( l > right_ ) error( "left, bad parameter resizing a Vrhomboid." );
   left_ = l;
   }
 
 
 void Vrhomboid::right( const int r )
   {
-  if( r < left_ ) error( "right, bad parameter resizing a Vrhomboid" );
+  if( r < left_ ) error( "right, bad parameter resizing a Vrhomboid." );
   right_ = r;
   }
 
 
 void Vrhomboid::height( const int h )
   {
-  if( h <= 0 ) error( "height, bad parameter resizing a Vrhomboid" );
+  if( h <= 0 ) error( "height, bad parameter resizing a Vrhomboid." );
   height_ = h;
   }
 
@@ -173,7 +173,7 @@ void Vrhomboid::height( const int h )
 void Vrhomboid::extend_left( const int l )
   {
   if( l > right_ )
-    error( "extend_left, bad parameter resizing a Vrhomboid" );
+    error( "extend_left, bad parameter resizing a Vrhomboid." );
   lvcenter_ = vcenter( l ); left_ = l;
   }
 
@@ -181,7 +181,7 @@ void Vrhomboid::extend_left( const int l )
 void Vrhomboid::extend_right( const int r )
   {
   if( r < left_ )
-    error( "extend_right, bad parameter resizing a Vrhomboid" );
+    error( "extend_right, bad parameter resizing a Vrhomboid." );
   rvcenter_ = vcenter( r ); right_ = r;
   }
 
@@ -217,8 +217,8 @@ bool Vrhomboid::includes( const int row, const int col ) const
 //
 void Track::set_track( const std::vector< Rectangle > & rectangle_vector )
   {
-  if( data.size() ) data.clear();
-  if( !rectangle_vector.size() ) return;
+  data.clear();
+  if( rectangle_vector.empty() ) return;
   std::vector< Rectangle > tmp;
   int max_gap = 0;
   bool last = false;
@@ -266,7 +266,7 @@ int Track::bottom( const int col ) const
   for( unsigned i = 0; i < data.size(); ++i )
     {
     const Vrhomboid & vr = data[i];
-    if( col <= vr.right() || i >= data.size() - 1 ) return vr.bottom( col );
+    if( col <= vr.right() || i + 1 >= data.size() ) return vr.bottom( col );
     }
   return 0;
   }
@@ -277,7 +277,7 @@ int Track::top( const int col ) const
   for( unsigned i = 0; i < data.size(); ++i )
     {
     const Vrhomboid & vr = data[i];
-    if( col <= vr.right() || i >= data.size() - 1 ) return vr.top( col );
+    if( col <= vr.right() || i + 1 >= data.size() ) return vr.top( col );
     }
   return 0;
   }
@@ -288,7 +288,7 @@ int Track::vcenter( const int col ) const
   for( unsigned i = 0; i < data.size(); ++i )
     {
     const Vrhomboid & vr = data[i];
-    if( col <= vr.right() || i >= data.size() - 1 ) return vr.vcenter( col );
+    if( col <= vr.right() || i + 1 >= data.size() ) return vr.vcenter( col );
     }
   return 0;
   }
@@ -298,7 +298,7 @@ bool Track::includes( const Rectangle & r ) const
   {
   for( unsigned i = 0; i < data.size(); ++i )
     if( data[i].includes( r ) ) return true;
-  if( !data.size() ) return false;
+  if( data.empty() ) return false;
   if( r.right() > data.back().right() )
     {
     Vrhomboid tmp = data.back();
@@ -319,7 +319,7 @@ bool Track::includes( const int row, const int col ) const
   {
   for( unsigned i = 0; i < data.size(); ++i )
     if( data[i].includes( row, col ) ) return true;
-  if( !data.size() ) return false;
+  if( data.empty() ) return false;
   if( col > data.back().right() )
     {
     Vrhomboid tmp = data.back();
